"""Integration tests for DnaerysClient against an in-process fake gRPC server."""

from __future__ import annotations

import warnings

import grpc
import pytest

from dnaerys import (
    DnaerysClient,
    DnaerysAuthenticationError,
    DnaerysConnectionError,
    DnaerysIncompleteResultWarning,
    DnaerysInvalidRequestError,
    DnaerysNotFoundError,
    DnaerysResourceExhausted,
    DnaerysServerError,
    Region,
    Bracket,
    Variant,
    VariantWithStats,
    VariantStream,
    VariantWithStatsStream,
    PaginatedQuery,
    CountResult,
    SamplesResult,
    HealthResult,
    ClusterNodesResult,
    DatasetInfo,
    PrsResult,
    SexMismatchResult,
    FstatXResult,
    KinshipResult,
    SampleKinshipResult,
    TopHweResult,
    TopChi2Result,
)
from dnaerys._proto import dnaerys_pb2 as pb2
from tests.conftest import (
    make_alleles_response,
    make_alleles_with_stats_response,
)


# ---------------------------------------------------------------------------
# Infrastructure methods
# ---------------------------------------------------------------------------


class TestInfrastructure:
    def test_health(self, client, fake_servicer):
        result = client.health()
        assert isinstance(result, HealthResult)
        assert result.status == "SERVING"

    def test_cluster_nodes(self, client, fake_servicer):
        result = client.cluster_nodes()
        assert isinstance(result, ClusterNodesResult)
        assert "node-1" in result.active_nodes
        assert result.total_nodes == 2

    def test_dataset_info(self, client, fake_servicer):
        result = client.dataset_info()
        assert isinstance(result, DatasetInfo)
        assert result.samples_total == 100
        assert len(result.cohorts) == 1
        assert result.cohorts[0].cohort_name == "test-cohort"
        assert len(result.prs) == 1

    def test_dataset_info_with_sample_names(self, client, fake_servicer):
        fake_servicer.dataset_info_response = pb2.DatasetInfoResponse(
            cohorts=[pb2.Cohort(
                cohort_name="test",
                samples_count=2,
                female_count=1,
                male_count=1,
                female_samples_names=["F1"],
                male_samples_names=["M1"],
            )],
            samples_total=2,
            assembly=pb2.GRCh38,
            elapsed_ms=5,
            node_id="node-1",
        )
        result = client.dataset_info(return_sample_names=True)
        assert result.cohorts[0].female_sample_names == ("F1",)
        assert result.cohorts[0].male_sample_names == ("M1",)


# ---------------------------------------------------------------------------
# Variant select — full call path
# ---------------------------------------------------------------------------


class TestSelectVariants:
    def test_select_variants_region(self, client, fake_servicer):
        stream = client.select_variants(region=Region("chr1", 100, 200))
        assert isinstance(stream, VariantStream)
        variants = stream.to_list()
        assert len(variants) == 2
        assert all(isinstance(v, Variant) for v in variants)

    def test_select_variants_region_in_samples(self, client, fake_servicer):
        stream = client.select_variants(
            region=Region("chr1", 100, 200), samples=["S1"],
        )
        variants = stream.to_list()
        assert len(variants) == 2

    def test_select_variants_bracket(self, client, fake_servicer):
        stream = client.select_variants(
            bracket=Bracket("chr1", 100, 200, 300, 400),
        )
        variants = stream.to_list()
        assert len(variants) == 2

    def test_select_variants_multi_regions(self, client, fake_servicer):
        stream = client.select_variants(
            regions=[Region("chr1", 100, 200), Region("chr2", 300, 400)],
        )
        variants = stream.to_list()
        assert len(variants) == 2

    def test_select_variants_with_stats_region_samples(self, client, fake_servicer):
        stream = client.select_variants_with_stats(
            region=Region("chr1", 100, 200), samples=["S1"],
        )
        assert isinstance(stream, VariantWithStatsStream)
        variants = stream.to_list()
        assert len(variants) == 2
        assert all(isinstance(v, VariantWithStats) for v in variants)

    def test_select_variants_with_stats_multi_regions(self, client, fake_servicer):
        stream = client.select_variants_with_stats(
            regions=[Region("chr1", 100, 200)],
        )
        variants = stream.to_list()
        assert len(variants) == 2


# ---------------------------------------------------------------------------
# Variant count
# ---------------------------------------------------------------------------


class TestCountVariants:
    def test_count_variants_region(self, client, fake_servicer):
        result = client.count_variants(region=Region("chr1", 100, 200))
        assert isinstance(result, CountResult)
        assert result.count == 42

    def test_count_variants_bracket(self, client, fake_servicer):
        result = client.count_variants(
            bracket=Bracket("chr1", 100, 200, 300, 400),
        )
        assert result.count == 42

    def test_count_variants_multi_regions_in_samples(self, client, fake_servicer):
        result = client.count_variants(
            regions=[Region("chr1", 100, 200)],
            samples=["S1"],
        )
        assert result.count == 42


# ---------------------------------------------------------------------------
# Sample queries
# ---------------------------------------------------------------------------


class TestSampleQueries:
    def test_select_samples_region(self, client, fake_servicer):
        result = client.select_samples(region=Region("chr1", 100, 200))
        assert isinstance(result, SamplesResult)
        assert "SAMPLE_A" in result.samples

    def test_count_samples_region(self, client, fake_servicer):
        result = client.count_samples(region=Region("chr1", 100, 200))
        assert isinstance(result, CountResult)
        assert result.count == 10

    def test_select_samples_hom_ref(self, client, fake_servicer):
        result = client.select_samples_hom_ref(chr="chr1", position=100)
        assert isinstance(result, SamplesResult)
        assert "SAMPLE_A" in result.samples

    def test_count_samples_hom_ref(self, client, fake_servicer):
        result = client.count_samples_hom_ref(chr="chr1", position=100)
        assert isinstance(result, CountResult)
        assert result.count == 10


# ---------------------------------------------------------------------------
# Inheritance
# ---------------------------------------------------------------------------


class TestInheritance:
    def test_select_de_novo(self, client, fake_servicer):
        stream = client.select_de_novo(
            parent1="P1", parent2="P2", proband="C1",
            region=Region("chr1", 100, 200),
        )
        assert isinstance(stream, VariantStream)
        assert len(stream.to_list()) == 2

    def test_select_het_dominant(self, client, fake_servicer):
        stream = client.select_het_dominant(
            affected_parent="AP", unaffected_parent="UP", affected_child="AC",
            region=Region("chr1", 100, 200),
        )
        assert isinstance(stream, VariantStream)
        assert len(stream.to_list()) == 2

    def test_select_hom_recessive(self, client, fake_servicer):
        stream = client.select_hom_recessive(
            unaffected_parent1="UP1", unaffected_parent2="UP2",
            affected_child="AC",
            region=Region("chr1", 100, 200),
        )
        assert isinstance(stream, VariantStream)
        assert len(stream.to_list()) == 2


# ---------------------------------------------------------------------------
# Statistics
# ---------------------------------------------------------------------------


class TestStatistics:
    def test_top_hwe(self, client, fake_servicer):
        result = client.top_hwe(n=10)
        assert isinstance(result, TopHweResult)
        assert len(result.variants) == 1

    def test_top_chi2(self, client, fake_servicer):
        result = client.top_chi2(n=10, samples=["S1"])
        assert isinstance(result, TopChi2Result)
        assert len(result.variants) == 1

    def test_prs(self, client, fake_servicer):
        result = client.prs(prs_name="PRS001", cohort_name="test-cohort")
        assert isinstance(result, PrsResult)
        assert result.prs_name == "PRS001"

    def test_sex_mismatch_check(self, client, fake_servicer):
        result = client.sex_mismatch_check(cohort_name="test-cohort")
        assert isinstance(result, SexMismatchResult)

    def test_fstat_x(self, client, fake_servicer):
        result = client.fstat_x(cohort_name="test-cohort")
        assert isinstance(result, FstatXResult)
        assert len(result.males) == 1


# ---------------------------------------------------------------------------
# Kinship
# ---------------------------------------------------------------------------


class TestKinship:
    def test_kinship(self, client, fake_servicer):
        result = client.kinship(cohort_name="test-cohort")
        assert isinstance(result, KinshipResult)
        assert len(result.pairs) == 1

    def test_kinship_duo(self, client, fake_servicer):
        result = client.kinship_duo(sample1="S1", sample2="S2")
        assert isinstance(result, KinshipResult)

    def test_kinship_trio(self, client, fake_servicer):
        result = client.kinship_trio(sample1="S1", sample2="S2", sample3="S3")
        assert isinstance(result, KinshipResult)

    def test_sample_kinship(self, client, fake_servicer):
        result = client.sample_kinship(sample_vcf="data.vcf")
        assert isinstance(result, SampleKinshipResult)
        assert result.accepted_snvs == 1000


# ---------------------------------------------------------------------------
# Streaming behaviour
# ---------------------------------------------------------------------------


class TestStreaming:
    def test_streaming_multiple_chunks(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=2),
            make_alleles_response(n_variants=3),
            make_alleles_response(n_variants=1),
        ]
        stream = client.select_variants(region=Region("chr1", 100, 200))
        variants = stream.to_list()
        assert len(variants) == 6

    def test_streaming_metadata_accumulation(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=1, elapsed_ms=10, node_id="n1"),
            make_alleles_response(n_variants=1, elapsed_ms=20, node_id="n2"),
        ]
        stream = client.select_variants(region=Region("chr1", 100, 200))
        stream.to_list()
        meta = stream.metadata
        assert meta.elapsed_ms == 20  # max
        assert set(meta.node_id.split(",")) == {"n1", "n2"}

    def test_streaming_to_list(self, client, fake_servicer):
        stream = client.select_variants(region=Region("chr1", 100, 200))
        result = stream.to_list()
        assert isinstance(result, list)
        assert all(isinstance(v, Variant) for v in result)

    def test_streaming_to_dataframe(self, client, fake_servicer):
        pytest.importorskip("pandas")
        stream = client.select_variants(region=Region("chr1", 100, 200))
        df = stream.to_dataframe()
        assert len(df) == 2
        assert "chr" in df.columns


# ---------------------------------------------------------------------------
# Incomplete cluster warning
# ---------------------------------------------------------------------------


class TestIncompleteClusterWarning:
    def test_unary_affected_warning(self, client, fake_servicer):
        fake_servicer.count_response = pb2.CountAllelesResponse(
            count=5, affected=True, elapsed_ms=5, node_id="n1",
        )
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            client.count_variants(region=Region("chr1", 100, 200))
        warning_types = [x.category for x in w]
        assert DnaerysIncompleteResultWarning in warning_types

    def test_streaming_affected_warning(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=1, affected=True),
        ]
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            stream = client.select_variants(region=Region("chr1", 100, 200))
            stream.to_list()
        warning_types = [x.category for x in w]
        assert DnaerysIncompleteResultWarning in warning_types

    def test_streaming_affected_warning_once(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=1, affected=True),
            make_alleles_response(n_variants=1, affected=True),
            make_alleles_response(n_variants=1, affected=True),
        ]
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            stream = client.select_variants(region=Region("chr1", 100, 200))
            stream.to_list()
        inc_warnings = [x for x in w if x.category is DnaerysIncompleteResultWarning]
        assert len(inc_warnings) == 1

    def test_unary_incomplete_not_affected_no_warning(self, client, fake_servicer):
        fake_servicer.count_response = pb2.CountAllelesResponse(
            count=5, incomplete_cluster=True, affected=False,
            elapsed_ms=5, node_id="n1",
        )
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            client.count_variants(region=Region("chr1", 100, 200))
        warning_types = [x.category for x in w]
        assert DnaerysIncompleteResultWarning not in warning_types


# ---------------------------------------------------------------------------
# Timeout
# ---------------------------------------------------------------------------


class TestTimeout:
    def test_timeout_raises_connection_error(self, fake_servicer, grpc_server):
        _, port, _ = grpc_server
        fake_servicer.sleep_seconds = 5.0
        with DnaerysClient(f"localhost:{port}", tls=False, default_timeout=0.1) as c:
            with pytest.raises(DnaerysConnectionError):
                c.count_variants(region=Region("chr1", 100, 200))


# ---------------------------------------------------------------------------
# Error propagation
# ---------------------------------------------------------------------------


class TestErrorPropagation:
    def test_invalid_argument_raises_invalid_request(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.INVALID_ARGUMENT
        with pytest.raises(DnaerysInvalidRequestError):
            client.count_variants(region=Region("chr1", 100, 200))

    def test_unavailable_raises_connection_error(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.UNAVAILABLE
        with pytest.raises(DnaerysConnectionError):
            client.count_variants(region=Region("chr1", 100, 200))

    def test_unauthenticated_raises_auth_error(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.UNAUTHENTICATED
        with pytest.raises(DnaerysAuthenticationError):
            client.count_variants(region=Region("chr1", 100, 200))

    def test_not_found_raises_not_found_error(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.NOT_FOUND
        with pytest.raises(DnaerysNotFoundError):
            client.count_variants(region=Region("chr1", 100, 200))

    def test_internal_raises_server_error(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.INTERNAL
        with pytest.raises(DnaerysServerError):
            client.count_variants(region=Region("chr1", 100, 200))

    def test_resource_exhausted_raises_resource_exhausted(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.RESOURCE_EXHAUSTED
        with pytest.raises(DnaerysResourceExhausted):
            client.count_variants(region=Region("chr1", 100, 200))

    def test_error_chaining(self, client, fake_servicer):
        fake_servicer.error_to_raise = grpc.StatusCode.INTERNAL
        with pytest.raises(DnaerysServerError) as exc_info:
            client.count_variants(region=Region("chr1", 100, 200))
        assert exc_info.value.__cause__ is not None


# ---------------------------------------------------------------------------
# Connection lifecycle
# ---------------------------------------------------------------------------


class TestConnectionLifecycle:
    def test_context_manager(self, fake_servicer, grpc_server):
        _, port, _ = grpc_server
        with DnaerysClient(f"localhost:{port}", tls=False) as c:
            result = c.health()
            assert result.status == "SERVING"

    def test_close(self, fake_servicer, grpc_server):
        _, port, _ = grpc_server
        c = DnaerysClient(f"localhost:{port}", tls=False)
        result = c.health()
        assert result.status == "SERVING"
        c.close()

    def test_tls_false(self, fake_servicer, grpc_server):
        _, port, _ = grpc_server
        with DnaerysClient(f"localhost:{port}", tls=False) as c:
            assert c.health().status == "SERVING"


# ---------------------------------------------------------------------------
# Validation
# ---------------------------------------------------------------------------


class TestValidation:
    def test_select_variants_no_location_raises(self, client, fake_servicer):
        with pytest.raises(DnaerysInvalidRequestError):
            client.select_variants()

    def test_select_variants_with_stats_single_region(self, client, fake_servicer):
        stream = client.select_variants_with_stats(region=Region("chr1", 100, 200))
        assert isinstance(stream, VariantWithStatsStream)
        variants = stream.to_list()
        assert len(variants) == 2

    def test_kinship_degree_and_threshold_raises(self, client, fake_servicer):
        with pytest.raises(DnaerysInvalidRequestError, match="mutually exclusive"):
            client.kinship(
                cohort_name="test", degree="FIRST_DEGREE", threshold=0.1,
            )

    def test_prs_no_cohort_no_samples_raises(self, client, fake_servicer):
        with pytest.raises(DnaerysInvalidRequestError, match="at least one"):
            client.prs(prs_name="PRS001")

    def test_default_hom_het_sent_to_server(self, client, fake_servicer):
        client.count_variants(region=Region("chr1", 100, 200))
        request = fake_servicer.last_request
        assert request.hom is True
        assert request.het is True


# ---------------------------------------------------------------------------
# Strong limit enforcement
# ---------------------------------------------------------------------------


class TestStrongLimit:
    def test_select_variants_limit_caps_results(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=5),
        ]
        stream = client.select_variants(
            region=Region("chr1", 100, 200), limit=1,
        )
        assert len(stream.to_list()) == 1

    def test_select_variants_limit_none_yields_all(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=5),
        ]
        stream = client.select_variants(
            region=Region("chr1", 100, 200), limit=None,
        )
        assert len(stream.to_list()) == 5

    def test_select_variants_with_stats_limit_caps_results(self, client, fake_servicer):
        fake_servicer.variant_with_stats_chunks = [
            make_alleles_with_stats_response(n_variants=5),
        ]
        stream = client.select_variants_with_stats(
            regions=[Region("chr1", 100, 200)], limit=2,
        )
        assert len(stream.to_list()) == 2


# ---------------------------------------------------------------------------
# Skip removal — TypeError when passing skip= to public methods
# ---------------------------------------------------------------------------


class TestSkipRemoval:
    def test_select_variants_rejects_skip(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.select_variants(
                region=Region("chr1", 100, 200), skip=5,
            )

    def test_select_variants_with_stats_rejects_skip(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.select_variants_with_stats(
                regions=[Region("chr1", 100, 200)], skip=5,
            )

    def test_count_samples_rejects_skip(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.count_samples(
                region=Region("chr1", 100, 200), skip=1,
            )

    def test_count_samples_rejects_limit(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.count_samples(
                region=Region("chr1", 100, 200), limit=1,
            )

    def test_select_de_novo_rejects_skip(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.select_de_novo(
                parent1="P1", parent2="P2", proband="C1",
                region=Region("chr1", 100, 200), skip=5,
            )

    def test_select_het_dominant_rejects_skip(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.select_het_dominant(
                affected_parent="AP", unaffected_parent="UP",
                affected_child="AC",
                region=Region("chr1", 100, 200), skip=5,
            )

    def test_select_hom_recessive_rejects_skip(self, client, fake_servicer):
        with pytest.raises(TypeError):
            client.select_hom_recessive(
                unaffected_parent1="UP1", unaffected_parent2="UP2",
                affected_child="AC",
                region=Region("chr1", 100, 200), skip=5,
            )


# ---------------------------------------------------------------------------
# Inheritance pagination
# ---------------------------------------------------------------------------


class TestInheritancePagination:
    def test_paginate_de_novo_returns_paginated_query(self, client, fake_servicer):
        q = client.paginate_de_novo(
            page_size=10,
            parent1="P1", parent2="P2", proband="C1",
            region=Region("chr1", 100, 200),
        )
        assert isinstance(q, PaginatedQuery)

    def test_paginate_de_novo_first_page(self, client, fake_servicer):
        fake_servicer.variant_chunks = [
            make_alleles_response(n_variants=4),
        ]
        q = client.paginate_de_novo(
            page_size=2,
            parent1="P1", parent2="P2", proband="C1",
            region=Region("chr1", 100, 200),
        )
        page = q.next_page()
        assert len(page.variants) == 2
        assert page.page_number == 1

    def test_paginate_het_dominant_returns_paginated_query(self, client, fake_servicer):
        q = client.paginate_het_dominant(
            page_size=10,
            affected_parent="AP", unaffected_parent="UP",
            affected_child="AC",
            region=Region("chr1", 100, 200),
        )
        assert isinstance(q, PaginatedQuery)

    def test_paginate_hom_recessive_returns_paginated_query(self, client, fake_servicer):
        q = client.paginate_hom_recessive(
            page_size=10,
            unaffected_parent1="UP1", unaffected_parent2="UP2",
            affected_child="AC",
            region=Region("chr1", 100, 200),
        )
        assert isinstance(q, PaginatedQuery)

    def test_paginate_variants_with_stats_returns_paginated_query(self, client, fake_servicer):
        q = client.paginate_variants_with_stats(
            page_size=10,
            regions=[Region("chr1", 100, 200)],
        )
        assert isinstance(q, PaginatedQuery)

    def test_paginate_variants_with_stats_first_page(self, client, fake_servicer):
        fake_servicer.variant_with_stats_chunks = [
            make_alleles_with_stats_response(n_variants=4),
        ]
        q = client.paginate_variants_with_stats(
            page_size=2,
            regions=[Region("chr1", 100, 200)],
        )
        page = q.next_page()
        assert len(page.variants) == 2
        assert page.page_number == 1



"""Tests for proto→dataclass and dataclass→proto conversion functions.

Each test constructs proto message objects directly using the generated
``dnaerys_pb2`` classes — no mocking of proto construction.
"""

from __future__ import annotations

from dnaerys._convert import (
    annotation_filter_to_proto,
    convert_alleles_with_stats_response_to_top_chi2,
    convert_alleles_with_stats_response_to_top_hwe,
    convert_cluster_nodes_response,
    convert_cohort,
    convert_count_alleles_response,
    convert_count_samples_response,
    convert_dataset_info_response,
    convert_fstat_x_response,
    convert_health_response,
    convert_kinship_response,
    convert_prs_info,
    convert_prs_response,
    convert_relatedness,
    convert_response_metadata,
    convert_sample_kinship_response,
    convert_sample_relatedness,
    convert_sample_score,
    convert_sample_stat,
    convert_samples_response,
    convert_sex_mismatch_response,
    convert_variant,
    convert_variant_with_stats,
)
from dnaerys._enums import (
    AlphaMissense,
    Chromosome,
    Consequence,
    Impact,
    KinshipDegree,
    RefAssembly,
    VariantType,
)
from dnaerys._proto import dnaerys_pb2 as pb2
from dnaerys._types import (
    AnnotationFilter,
    ClusterNodesResult,
    Cohort,
    CountResult,
    DatasetInfo,
    FstatXResult,
    HealthResult,
    KinshipResult,
    PrsInfo,
    PrsResult,
    Relatedness,
    ResponseMetadata,
    SampleKinshipResult,
    SampleRelatedness,
    SampleScore,
    SamplesResult,
    SampleStat,
    SexMismatchResult,
    TopChi2Result,
    TopHweResult,
    Variant,
    VariantWithStats,
)


# ---------------------------------------------------------------------------
# Variant conversion
# ---------------------------------------------------------------------------


class TestConvertVariant:
    def test_convert_variant_basic(self):
        proto = pb2.Variant(
            chr=1,
            start=100,
            end=100,
            ref="A",
            alt="T",
            af=0.5,
            ac=1.0,
            an=2,
            homc=0,
            hetc=1,
            misc=0,
            homfc=0,
            hetfc=0,
            misfc=0,
            gnomADe=0.1,
            gnomADg=0.2,
            cadd_raw=3.5,
            cadd_phred=15.0,
            am_score=0.8,
            amino_acids="A/V",
            biallelic=True,
        )
        v = convert_variant(proto)
        assert isinstance(v, Variant)
        assert v.chr == Chromosome.CHR_1
        assert v.start == 100
        assert v.end == 100
        assert v.ref == "A"
        assert v.alt == "T"
        assert v.af == 0.5
        assert v.ac == 1.0
        assert v.an == 2
        assert v.homc == 0
        assert v.hetc == 1
        assert v.misc == 0
        assert v.homfc == 0
        assert v.hetfc == 0
        assert v.misfc == 0
        assert abs(v.gnomad_exomes_af - 0.1) < 1e-6
        assert abs(v.gnomad_genomes_af - 0.2) < 1e-6
        assert abs(v.cadd_raw - 3.5) < 1e-6
        assert abs(v.cadd_phred - 15.0) < 1e-6
        assert abs(v.am_score - 0.8) < 1e-6
        assert v.amino_acids == "A/V"
        assert v.biallelic is True

    def test_convert_variant_chromosome_is_enum(self):
        proto = pb2.Variant(chr=23, start=1, end=1, ref="G", alt="C")
        v = convert_variant(proto)
        assert isinstance(v.chr, Chromosome)
        assert v.chr == Chromosome.CHR_X

    def test_convert_variant_gnomad_field_mapping(self):
        proto = pb2.Variant(
            chr=1, start=1, end=1, ref="A", alt="T",
            gnomADe=0.5, gnomADg=0.3,
        )
        v = convert_variant(proto)
        assert abs(v.gnomad_exomes_af - 0.5) < 1e-6
        assert abs(v.gnomad_genomes_af - 0.3) < 1e-6

    def test_convert_variant_zero_annotation_fields(self):
        proto = pb2.Variant(
            chr=1, start=1, end=1, ref="A", alt="T",
            gnomADe=0.0, gnomADg=0.0, cadd_raw=0.0, cadd_phred=0.0, am_score=0.0,
        )
        v = convert_variant(proto)
        assert v.gnomad_exomes_af == 0.0
        assert v.gnomad_genomes_af == 0.0
        assert v.cadd_raw == 0.0
        assert v.cadd_phred == 0.0
        assert v.am_score == 0.0


# ---------------------------------------------------------------------------
# VariantWithStats conversion
# ---------------------------------------------------------------------------


class TestConvertVariantWithStats:
    def _make_proto(self, **overrides):
        variant_kwargs = {
            "chr": 1, "start": 200, "end": 200, "ref": "G", "alt": "C",
            "af": 0.3, "ac": 2.0, "an": 6, "homc": 1, "hetc": 2,
            "misc": 0, "homfc": 0, "hetfc": 1, "misfc": 0,
            "gnomADe": 0.15, "gnomADg": 0.25,
            "cadd_raw": 1.5, "cadd_phred": 10.0, "am_score": 0.6,
            "amino_acids": "R/Q", "biallelic": True,
        }
        stats_kwargs = {
            "vaf": 0.4, "vac": 1.5, "van": 4, "vhomc": 0, "vhetc": 1,
            "vhomfc": 0, "vhetfc": 0,
            "phwe": 0.05, "pchi2": 0.01, "ibc": 0.02,
        }
        # The 'or' field must be set via **kwargs
        or_val = overrides.pop("or_val", 1.5)
        stats_kwargs.update(overrides)
        return pb2.VariantWithStats(
            variant=pb2.Variant(**variant_kwargs),
            **stats_kwargs,
            **{"or": or_val},
        )

    def test_convert_variant_with_stats_flattened(self):
        proto = self._make_proto()
        v = convert_variant_with_stats(proto)
        # Variant fields are flattened — accessible directly, not via .variant
        assert v.chr == Chromosome.CHR_1
        assert v.start == 200
        assert v.ref == "G"
        assert v.alt == "C"

    def test_convert_variant_with_stats_all_fields(self):
        proto = self._make_proto()
        v = convert_variant_with_stats(proto)
        assert isinstance(v, VariantWithStats)
        # All 21 variant fields
        assert v.chr == Chromosome.CHR_1
        assert v.start == 200
        assert v.end == 200
        assert v.ref == "G"
        assert v.alt == "C"
        assert abs(v.af - 0.3) < 1e-6
        assert abs(v.ac - 2.0) < 1e-6
        assert v.an == 6
        assert v.homc == 1
        assert v.hetc == 2
        assert v.misc == 0
        assert v.homfc == 0
        assert v.hetfc == 1
        assert v.misfc == 0
        assert abs(v.gnomad_exomes_af - 0.15) < 1e-6
        assert abs(v.gnomad_genomes_af - 0.25) < 1e-6
        assert abs(v.cadd_raw - 1.5) < 1e-6
        assert abs(v.cadd_phred - 10.0) < 1e-6
        assert abs(v.am_score - 0.6) < 1e-6
        assert v.amino_acids == "R/Q"
        assert v.biallelic is True
        # 11 stats fields
        assert abs(v.vaf - 0.4) < 1e-6
        assert abs(v.vac - 1.5) < 1e-6
        assert v.van == 4
        assert v.vhomc == 0
        assert v.vhetc == 1
        assert v.vhomfc == 0
        assert v.vhetfc == 0
        assert abs(v.phwe - 0.05) < 1e-6
        assert abs(v.pchi2 - 0.01) < 1e-6
        assert abs(v.odds_ratio - 1.5) < 1e-6
        assert abs(v.ibc - 0.02) < 1e-6

    def test_convert_variant_with_stats_odds_ratio_mapping(self):
        # Verify proto field 'or' maps to odds_ratio
        proto = self._make_proto(or_val=2.75)
        v = convert_variant_with_stats(proto)
        assert abs(v.odds_ratio - 2.75) < 1e-6


# ---------------------------------------------------------------------------
# Response metadata
# ---------------------------------------------------------------------------


class TestConvertResponseMetadata:
    def test_convert_response_metadata_basic(self):
        proto = pb2.AllelesResponse(
            incomplete_cluster=True,
            affected=True,
            elapsed_ms=150,
            elapsed_db_ms=80,
            node_id="node-1",
        )
        m = convert_response_metadata(proto)
        assert isinstance(m, ResponseMetadata)
        assert m.elapsed_ms == 150
        assert m.elapsed_db_ms == 80
        assert m.node_id == "node-1"
        assert m.incomplete_cluster is True
        assert m.affected is True

    def test_convert_response_metadata_no_affected(self):
        # KinshipResponse has no 'affected' field — should default to False
        proto = pb2.KinshipResponse(
            incomplete_cluster=True,
            elapsed_ms=200,
            elapsed_db_ms=100,
            node_id="node-2",
        )
        m = convert_response_metadata(proto)
        assert m.affected is False
        assert m.incomplete_cluster is True
        assert m.elapsed_ms == 200


# ---------------------------------------------------------------------------
# Count responses
# ---------------------------------------------------------------------------


class TestConvertCountResponses:
    def test_convert_count_alleles_response(self):
        proto = pb2.CountAllelesResponse(
            count=42,
            incomplete_cluster=False,
            affected=False,
            elapsed_ms=50,
            elapsed_db_ms=30,
            node_id="n1",
        )
        result = convert_count_alleles_response(proto)
        assert isinstance(result, CountResult)
        assert result.count == 42
        assert result.metadata.elapsed_ms == 50
        assert result.metadata.node_id == "n1"

    def test_convert_count_samples_response(self):
        proto = pb2.CountSamplesResponse(
            count=10,
            incomplete_cluster=False,
            affected=False,
            elapsed_ms=25,
            elapsed_db_ms=15,
            node_id="n2",
        )
        result = convert_count_samples_response(proto)
        assert isinstance(result, CountResult)
        assert result.count == 10
        assert result.metadata.elapsed_ms == 25


# ---------------------------------------------------------------------------
# Samples response
# ---------------------------------------------------------------------------


class TestConvertSamplesResponse:
    def test_convert_samples_response(self):
        proto = pb2.SamplesResponse(
            samples=["sample1", "sample2", "sample3"],
            incomplete_cluster=False,
            affected=False,
            elapsed_ms=30,
            elapsed_db_ms=20,
            node_id="n1",
        )
        result = convert_samples_response(proto)
        assert isinstance(result, SamplesResult)
        assert result.samples == ("sample1", "sample2", "sample3")
        assert isinstance(result.samples, tuple)
        assert result.metadata.elapsed_ms == 30


# ---------------------------------------------------------------------------
# Infrastructure responses
# ---------------------------------------------------------------------------


class TestConvertInfrastructure:
    def test_convert_health_response(self):
        proto = pb2.HealthResponse(status="ok")
        result = convert_health_response(proto)
        assert isinstance(result, HealthResult)
        assert result.status == "ok"

    def test_convert_cluster_nodes_response(self):
        proto = pb2.ClusterNodesResponse(
            active_nodes=["node-a", "node-b"],
            inactive_nodes=["node-c"],
            total_nodes=3,
            elapsed_ms=10,
        )
        result = convert_cluster_nodes_response(proto)
        assert isinstance(result, ClusterNodesResult)
        assert result.active_nodes == ("node-a", "node-b")
        assert result.inactive_nodes == ("node-c",)
        assert result.total_nodes == 3
        assert result.elapsed_ms == 10

    def test_convert_dataset_info_response(self):
        cohort_proto = pb2.Cohort(
            cohort_name="test-cohort",
            samples_count=100,
            female_count=50,
            male_count=50,
            female_samples_names=["f1", "f2"],
            male_samples_names=["m1", "m2"],
            synthetic=False,
        )
        prs_proto = pb2.PRS(name="prs1", desc="Test PRS", cardinality=500)
        proto = pb2.DatasetInfoResponse(
            cohorts=[cohort_proto],
            samples_total=100,
            females_total=50,
            males_total=50,
            variants_total=1000000,
            assembly=2,  # GRCh38
            rto=False,
            prs=[prs_proto],
            timestamp="2024-01-01T00:00:00Z",
            data_format=1,
            notes="test notes",
            rings_total=4,
            elapsed_ms=100,
            node_id="node-1",
        )
        result = convert_dataset_info_response(proto)
        assert isinstance(result, DatasetInfo)
        assert len(result.cohorts) == 1
        assert result.cohorts[0].cohort_name == "test-cohort"
        assert result.samples_total == 100
        assert result.assembly == RefAssembly.GRCh38
        assert len(result.prs) == 1
        assert result.prs[0].name == "prs1"
        assert result.prs[0].description == "Test PRS"
        assert result.elapsed_ms == 100
        assert result.node_id == "node-1"

    def test_convert_cohort(self):
        proto = pb2.Cohort(
            cohort_name="cohort-A",
            samples_count=200,
            female_count=110,
            male_count=90,
            female_samples_names=["fa", "fb"],
            male_samples_names=["ma"],
            synthetic=True,
        )
        result = convert_cohort(proto)
        assert isinstance(result, Cohort)
        assert result.cohort_name == "cohort-A"
        assert result.samples_count == 200
        assert result.female_count == 110
        assert result.male_count == 90
        assert result.female_sample_names == ("fa", "fb")
        assert result.male_sample_names == ("ma",)
        assert result.synthetic is True

    def test_convert_prs_info(self):
        proto = pb2.PRS(name="prs-abc", desc="A test PRS", cardinality=250)
        result = convert_prs_info(proto)
        assert isinstance(result, PrsInfo)
        assert result.name == "prs-abc"
        assert result.description == "A test PRS"
        assert result.cardinality == 250


# ---------------------------------------------------------------------------
# Statistical responses
# ---------------------------------------------------------------------------


class TestConvertStatistical:
    def test_convert_prs_response(self):
        score = pb2.SampleScore(
            sample="s1", scores_sum=1.5,
            hethom_cardinality=10, ref_cardinality=5,
            mis_cardinality=2, imputed_sum=0.3,
        )
        proto = pb2.PRSResponse(
            prs_name="prs1",
            sample_scores=[score],
            dominant=True,
            recessive=False,
            prs_cardinality=500,
            incomplete_cluster=False,
            elapsed_ms=100,
            elapsed_db_ms=80,
            node_id="n1",
        )
        result = convert_prs_response(proto)
        assert isinstance(result, PrsResult)
        assert result.prs_name == "prs1"
        assert len(result.sample_scores) == 1
        assert result.sample_scores[0].sample == "s1"
        assert result.dominant is True
        assert result.recessive is False
        assert result.prs_cardinality == 500
        assert result.metadata.elapsed_ms == 100

    def test_convert_sample_score(self):
        proto = pb2.SampleScore(
            sample="s-test",
            scores_sum=2.5,
            hethom_cardinality=15,
            ref_cardinality=8,
            mis_cardinality=3,
            imputed_sum=0.7,
        )
        result = convert_sample_score(proto)
        assert isinstance(result, SampleScore)
        assert result.sample == "s-test"
        assert abs(result.scores_sum - 2.5) < 1e-6
        assert result.hethom_cardinality == 15
        assert result.ref_cardinality == 8
        assert result.mis_cardinality == 3
        assert abs(result.imputed_sum - 0.7) < 1e-6

    def test_convert_sex_mismatch_response(self):
        male = pb2.SampleStat(
            sample="m1", reported_sex="male", observed_sex="female", f_stat=0.3,
        )
        female = pb2.SampleStat(
            sample="f1", reported_sex="female", observed_sex="male", f_stat=0.9,
        )
        proto = pb2.SexMismatchResponse(
            mismatch_males=[male],
            mismatch_females=[female],
            incomplete_cluster=False,
            elapsed_ms=50,
            elapsed_db_ms=30,
            node_id="n1",
        )
        result = convert_sex_mismatch_response(proto)
        assert isinstance(result, SexMismatchResult)
        assert len(result.mismatch_males) == 1
        assert result.mismatch_males[0].sample == "m1"
        assert len(result.mismatch_females) == 1
        assert result.mismatch_females[0].sample == "f1"
        assert result.metadata.elapsed_ms == 50

    def test_convert_sample_stat(self):
        proto = pb2.SampleStat(
            sample="sample-x",
            reported_sex="male",
            observed_sex="female",
            f_stat=0.42,
        )
        result = convert_sample_stat(proto)
        assert isinstance(result, SampleStat)
        assert result.sample == "sample-x"
        assert result.reported_sex == "male"
        assert result.observed_sex == "female"
        assert abs(result.f_stat - 0.42) < 1e-6

    def test_convert_fstat_x_response(self):
        m = pb2.SampleStat(sample="m1", reported_sex="male", observed_sex="male", f_stat=0.95)
        f = pb2.SampleStat(sample="f1", reported_sex="female", observed_sex="female", f_stat=0.1)
        proto = pb2.FstatXResponse(
            males=[m], females=[f],
            incomplete_cluster=False,
            elapsed_ms=40, elapsed_db_ms=25, node_id="n1",
        )
        result = convert_fstat_x_response(proto)
        assert isinstance(result, FstatXResult)
        assert len(result.males) == 1
        assert len(result.females) == 1
        assert result.males[0].sample == "m1"
        assert result.metadata.elapsed_ms == 40

    def test_convert_kinship_response(self):
        rel = pb2.Relatedness(
            sample1="s1", sample2="s2",
            degree=2,  # FIRST_DEGREE
            phi_bwf=0.25,
        )
        proto = pb2.KinshipResponse(
            rel=[rel],
            incomplete_cluster=False,
            elapsed_ms=60, elapsed_db_ms=40, node_id="n1",
        )
        result = convert_kinship_response(proto)
        assert isinstance(result, KinshipResult)
        assert len(result.pairs) == 1
        assert result.pairs[0].sample1 == "s1"
        assert result.pairs[0].sample2 == "s2"
        assert result.metadata.elapsed_ms == 60

    def test_convert_relatedness(self):
        proto = pb2.Relatedness(
            sample1="alice", sample2="bob",
            degree=1,  # TWINS_MONOZYGOTIC
            phi_bwf=0.48,
        )
        result = convert_relatedness(proto)
        assert isinstance(result, Relatedness)
        assert result.sample1 == "alice"
        assert result.sample2 == "bob"
        assert result.degree == KinshipDegree.TWINS_MONOZYGOTIC
        assert isinstance(result.degree, KinshipDegree)
        assert abs(result.phi_bwf - 0.48) < 1e-6

    def test_convert_sample_kinship_response(self):
        rel = pb2.RelatednessPerSample(
            sample="ds1", degree=3, phi_bwf=0.1,
            common_loci=5000,
            nHetS1=100, nHetS2=120, nHetS1S2=50, nHomOp=10,
        )
        proto = pb2.SampleKinshipResponse(
            rel=[rel],
            accepted_snvs=10000,
            incomplete_cluster=False,
            elapsed_ms=70, elapsed_db_ms=50, node_id="n1",
        )
        result = convert_sample_kinship_response(proto)
        assert isinstance(result, SampleKinshipResult)
        assert len(result.relatives) == 1
        assert result.accepted_snvs == 10000
        assert result.metadata.elapsed_ms == 70

    def test_convert_sample_relatedness(self):
        proto = pb2.RelatednessPerSample(
            sample="ds-sample",
            degree=2,  # FIRST_DEGREE
            phi_bwf=0.22,
            common_loci=8000,
            nHetS1=200,
            nHetS2=180,
            nHetS1S2=90,
            nHomOp=15,
        )
        result = convert_sample_relatedness(proto)
        assert isinstance(result, SampleRelatedness)
        assert result.sample == "ds-sample"
        assert result.degree == KinshipDegree.FIRST_DEGREE
        assert abs(result.phi_bwf - 0.22) < 1e-6
        assert result.common_loci == 8000
        assert result.n_het_s1 == 200
        assert result.n_het_s2 == 180
        assert result.n_het_s1s2 == 90
        assert result.n_hom_op == 15

    def test_convert_top_hwe_result(self):
        vws = pb2.VariantWithStats(
            variant=pb2.Variant(
                chr=1, start=100, end=100, ref="A", alt="T",
                af=0.5, ac=1.0, an=2, homc=0, hetc=1,
                gnomADe=0.1, gnomADg=0.2,
                amino_acids="X/Y", biallelic=True,
            ),
            vaf=0.3, vac=1.0, van=4, vhomc=0, vhetc=1,
            phwe=0.001, pchi2=0.05, ibc=0.01,
            **{"or": 1.2},
        )
        proto = pb2.AllelesWithStatsResponse(
            variants=[vws],
            incomplete_cluster=False, affected=False,
            elapsed_ms=100, elapsed_db_ms=80, node_id="n1",
        )
        result = convert_alleles_with_stats_response_to_top_hwe(proto)
        assert isinstance(result, TopHweResult)
        assert len(result.variants) == 1
        assert result.variants[0].chr == Chromosome.CHR_1
        assert result.metadata.elapsed_ms == 100

    def test_convert_top_chi2_result(self):
        vws = pb2.VariantWithStats(
            variant=pb2.Variant(
                chr=2, start=500, end=500, ref="C", alt="G",
                af=0.1, ac=0.5, an=10, homc=0, hetc=1,
                gnomADe=0.0, gnomADg=0.0,
                amino_acids="", biallelic=True,
            ),
            vaf=0.2, vac=0.5, van=5, vhomc=0, vhetc=1,
            phwe=0.5, pchi2=0.003, ibc=0.0,
            **{"or": 3.5},
        )
        proto = pb2.AllelesWithStatsResponse(
            variants=[vws],
            incomplete_cluster=False, affected=False,
            elapsed_ms=200, elapsed_db_ms=150, node_id="n2",
        )
        result = convert_alleles_with_stats_response_to_top_chi2(proto)
        assert isinstance(result, TopChi2Result)
        assert len(result.variants) == 1
        assert result.variants[0].chr == Chromosome.CHR_2
        assert abs(result.variants[0].odds_ratio - 3.5) < 1e-6
        assert result.metadata.elapsed_ms == 200


# ---------------------------------------------------------------------------
# AnnotationFilter → proto
# ---------------------------------------------------------------------------


class TestAnnotationFilterToProto:
    def test_annotation_filter_to_proto_empty(self):
        af = AnnotationFilter()
        proto = annotation_filter_to_proto(af)
        assert isinstance(proto, pb2.Annotations)
        # All defaults — repeated fields empty, floats 0.0, bools False
        assert len(proto.consequence) == 0
        assert len(proto.impact) == 0
        assert proto.af_lt == 0.0
        assert proto.biallelicOnly is False

    def test_annotation_filter_to_proto_consequence(self):
        af = AnnotationFilter(consequence=[Consequence.MISSENSE_VARIANT])
        proto = annotation_filter_to_proto(af)
        assert len(proto.consequence) == 1
        assert proto.consequence[0] == int(Consequence.MISSENSE_VARIANT)

    def test_annotation_filter_to_proto_multiple_impacts(self):
        af = AnnotationFilter(impact=[Impact.HIGH, Impact.MODERATE])
        proto = annotation_filter_to_proto(af)
        assert len(proto.impact) == 2
        assert int(Impact.HIGH) in proto.impact
        assert int(Impact.MODERATE) in proto.impact

    def test_annotation_filter_to_proto_float_fields(self):
        af = AnnotationFilter(af_lt=0.01, gnomad_exomes_af_gt=0.001)
        proto = annotation_filter_to_proto(af)
        assert abs(proto.af_lt - 0.01) < 1e-6
        assert abs(proto.gnomad_exomes_af_gt - 0.001) < 1e-6
        # Unset fields remain at proto default 0.0
        assert proto.af_gt == 0.0

    def test_annotation_filter_to_proto_bool_fields(self):
        af = AnnotationFilter(biallelic_only=True, exclude_males=True)
        proto = annotation_filter_to_proto(af)
        assert proto.biallelicOnly is True
        assert proto.excludeMales is True
        assert proto.multiallelicOnly is False
        assert proto.excludeFemales is False

    def test_annotation_filter_to_proto_all_fields(self):
        af = AnnotationFilter(
            variant_type=[VariantType.SNV],
            consequence=[Consequence.MISSENSE_VARIANT],
            impact=[Impact.HIGH],
            clin_significance=[],
            af_lt=0.05,
            af_gt=0.001,
            gnomad_exomes_af_lt=0.1,
            gnomad_exomes_af_gt=0.0001,
            gnomad_genomes_af_lt=0.2,
            gnomad_genomes_af_gt=0.0002,
            cadd_raw_lt=5.0,
            cadd_raw_gt=1.0,
            cadd_phred_lt=30.0,
            cadd_phred_gt=10.0,
            am_score_lt=0.9,
            am_score_gt=0.1,
            biallelic_only=True,
            exclude_females=True,
        )
        proto = annotation_filter_to_proto(af)
        assert len(proto.variant_type) == 1
        assert proto.variant_type[0] == int(VariantType.SNV)
        assert len(proto.consequence) == 1
        assert len(proto.impact) == 1
        assert abs(proto.af_lt - 0.05) < 1e-6
        assert abs(proto.af_gt - 0.001) < 1e-6
        assert abs(proto.gnomad_exomes_af_lt - 0.1) < 1e-6
        assert abs(proto.gnomad_genomes_af_lt - 0.2) < 1e-6
        assert abs(proto.cadd_raw_lt - 5.0) < 1e-6
        assert abs(proto.cadd_phred_gt - 10.0) < 1e-6
        assert abs(proto.am_score_lt - 0.9) < 1e-6
        assert abs(proto.am_score_gt - 0.1) < 1e-6
        assert proto.biallelicOnly is True
        assert proto.excludeFemales is True
        assert proto.multiallelicOnly is False
        assert proto.excludeMales is False



"""Thread safety tests for DnaerysClient."""

from __future__ import annotations

from concurrent import futures

import pytest

from dnaerys import DnaerysClient, Region
from tests.conftest import make_alleles_response


class TestThreadSafety:
    def test_concurrent_unary_calls(self, client, fake_servicer):
        """10 threads each calling count_variants, all return correct result."""
        def call_count(_):
            return client.count_variants(region=Region("chr1", 100, 200))

        with futures.ThreadPoolExecutor(max_workers=10) as pool:
            results = list(pool.map(call_count, range(10)))
        assert all(r.count == 42 for r in results)

    def test_concurrent_streaming_calls(self, client, fake_servicer):
        """5 threads each iterating select_variants, no cross-contamination."""
        fake_servicer.variant_chunks = [make_alleles_response(n_variants=3)]

        def call_select(_):
            stream = client.select_variants(region=Region("chr1", 100, 200))
            return stream.to_list()

        with futures.ThreadPoolExecutor(max_workers=5) as pool:
            results = list(pool.map(call_select, range(5)))
        assert all(len(variants) == 3 for variants in results)

    def test_concurrent_mixed_calls(self, client, fake_servicer):
        """Mix of count_variants and select_variants from thread pool."""
        fake_servicer.variant_chunks = [make_alleles_response(n_variants=2)]

        def mixed_call(i):
            if i % 2 == 0:
                return client.count_variants(region=Region("chr1", 100, 200))
            else:
                stream = client.select_variants(region=Region("chr1", 100, 200))
                return stream.to_list()

        with futures.ThreadPoolExecutor(max_workers=8) as pool:
            results = list(pool.map(mixed_call, range(8)))
        for i, r in enumerate(results):
            if i % 2 == 0:
                assert r.count == 42
            else:
                assert len(r) == 2

    def test_client_used_after_close(self, grpc_server):
        """Call method after client.close() raises appropriate error."""
        _, port, _ = grpc_server
        c = DnaerysClient(f"localhost:{port}", tls=False)
        c.close()
        with pytest.raises(Exception):
            c.health()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: dnaerys.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'dnaerys.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rdnaerys.proto\x12\x18org.dnaerys.cluster.grpc\"\x0f\n\rHealthRequest\" \n\x0eHealthResponse\x12\x0e\n\x06status\x18\x01 \x01(\t\"\x15\n\x13\x43lusterNodesRequest\"m\n\x14\x43lusterNodesResponse\x12\x14\n\x0c\x61\x63tive_nodes\x18\x01 \x03(\t\x12\x16\n\x0einactive_nodes\x18\x02 \x03(\t\x12\x13\n\x0btotal_nodes\x18\x03 \x01(\x05\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\"2\n\x12\x44\x61tasetInfoRequest\x12\x1c\n\x14return_samples_names\x18\x01 \x01(\x08\"\x86\x03\n\x13\x44\x61tasetInfoResponse\x12\x31\n\x07\x63ohorts\x18\x01 \x03(\x0b\x32 .org.dnaerys.cluster.grpc.Cohort\x12\x15\n\rsamples_total\x18\x02 \x01(\x05\x12\x15\n\rfemales_total\x18\x03 \x01(\x05\x12\x13\n\x0bmales_total\x18\x04 \x01(\x05\x12\x16\n\x0evariants_total\x18\x05 \x01(\x05\x12\x37\n\x08\x61ssembly\x18\x06 \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x0b\n\x03rto\x18\x07 \x01(\x08\x12*\n\x03prs\x18\x08 \x03(\x0b\x32\x1d.org.dnaerys.cluster.grpc.PRS\x12\x11\n\ttimestamp\x18\t \x01(\t\x12\x13\n\x0b\x64\x61ta_format\x18\n \x01(\x05\x12\r\n\x05notes\x18\x0b \x01(\t\x12\x13\n\x0brings_total\x18\x0c \x01(\x05\x12\x12\n\nelapsed_ms\x18\x0e \x01(\x03\x12\x0f\n\x07node_id\x18\x0f \x01(\t\"\xab\x01\n\x06\x43ohort\x12\x13\n\x0b\x63ohort_name\x18\x01 \x01(\t\x12\x15\n\rsamples_count\x18\x02 \x01(\x05\x12\x14\n\x0c\x66\x65male_count\x18\x03 \x01(\x05\x12\x12\n\nmale_count\x18\x04 \x01(\x05\x12\x1c\n\x14\x66\x65male_samples_names\x18\x05 \x03(\t\x12\x1a\n\x12male_samples_names\x18\x06 \x03(\t\x12\x11\n\tsynthetic\x18\x07 \x01(\x08\"6\n\x03PRS\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x65sc\x18\x02 \x01(\t\x12\x13\n\x0b\x63\x61rdinality\x18\x03 \x01(\x05\"\xfb\x06\n\x0b\x41nnotations\x12;\n\x0cvariant_type\x18\x01 \x03(\x0e\x32%.org.dnaerys.cluster.grpc.VariantType\x12;\n\x0c\x66\x65\x61ture_type\x18\x02 \x03(\x0e\x32%.org.dnaerys.cluster.grpc.FeatureType\x12\x33\n\x08\x62io_type\x18\x03 \x03(\x0e\x32!.org.dnaerys.cluster.grpc.BioType\x12:\n\x0b\x63onsequence\x18\x04 \x03(\x0e\x32%.org.dnaerys.cluster.grpc.Consequence\x12\x30\n\x06impact\x18\x05 \x03(\x0e\x32 .org.dnaerys.cluster.grpc.Impact\x12;\n\x07\x63linsgn\x18\x06 \x03(\x0e\x32*.org.dnaerys.cluster.grpc.ClinSignificance\x12\r\n\x05\x61\x66_lt\x18\x07 \x01(\x02\x12\r\n\x05\x61\x66_gt\x18\x08 \x01(\x02\x12\x1b\n\x13gnomad_exomes_af_lt\x18\t \x01(\x02\x12\x1b\n\x13gnomad_exomes_af_gt\x18\n \x01(\x02\x12\x1c\n\x14gnomad_genomes_af_lt\x18\x0b \x01(\x02\x12\x1c\n\x14gnomad_genomes_af_gt\x18\x0c \x01(\x02\x12,\n\x04sift\x18\r \x03(\x0e\x32\x1e.org.dnaerys.cluster.grpc.SIFT\x12\x34\n\x08polyphen\x18\x0e \x03(\x0e\x32\".org.dnaerys.cluster.grpc.PolyPhen\x12\x13\n\x0b\x63\x61\x64\x64_raw_lt\x18\x0f \x01(\x02\x12\x13\n\x0b\x63\x61\x64\x64_raw_gt\x18\x10 \x01(\x02\x12\x15\n\rcadd_phred_lt\x18\x11 \x01(\x02\x12\x15\n\rcadd_phred_gt\x18\x12 \x01(\x02\x12\x13\n\x0b\x61m_score_lt\x18\x13 \x01(\x02\x12\x13\n\x0b\x61m_score_gt\x18\x14 \x01(\x02\x12\x39\n\x08\x61m_class\x18\x15 \x03(\x0e\x32\'.org.dnaerys.cluster.grpc.AlphaMissense\x12\x15\n\rbiallelicOnly\x18\x16 \x01(\x08\x12\x18\n\x10multiallelicOnly\x18\x17 \x01(\x08\x12\x14\n\x0c\x65xcludeMales\x18\x18 \x01(\x08\x12\x16\n\x0e\x65xcludeFemales\x18\x19 \x01(\x08\"\xd9\x02\n\x16\x41llelesInRegionRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0b\n\x03ref\x18\x04 \x01(\t\x12\x0b\n\x03\x61lt\x18\x05 \x01(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\x12\x0c\n\x04skip\x18\x0c \x01(\x05\x12\r\n\x05limit\x18\r \x01(\x05\"\x86\x03\n\x17\x41llelesInBracketRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\x11\n\tstart_min\x18\x02 \x01(\x05\x12\x11\n\tstart_max\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nd_min\x18\x04 \x01(\x05\x12\x0f\n\x07\x65nd_max\x18\x05 \x01(\x05\x12\x0b\n\x03ref\x18\x06 \x01(\t\x12\x0b\n\x03\x61lt\x18\x07 \x01(\t\x12\x0b\n\x03hom\x18\x08 \x01(\x08\x12\x0b\n\x03het\x18\t \x01(\x08\x12\x32\n\x03\x61nn\x18\n \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\x0b \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0c \x01(\x05\x12\x18\n\x10variantMaxLength\x18\r \x01(\x05\x12\x0c\n\x04skip\x18\x0e \x01(\x05\x12\r\n\x05limit\x18\x0f \x01(\x05\"\xa0\x03\n AllelesInBracketInSamplesRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\x11\n\tstart_min\x18\x02 \x01(\x05\x12\x11\n\tstart_max\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nd_min\x18\x04 \x01(\x05\x12\x0f\n\x07\x65nd_max\x18\x05 \x01(\x05\x12\x0b\n\x03ref\x18\x06 \x01(\t\x12\x0b\n\x03\x61lt\x18\x07 \x01(\t\x12\x0b\n\x03hom\x18\x08 \x01(\x08\x12\x0b\n\x03het\x18\t \x01(\x08\x12\x32\n\x03\x61nn\x18\n \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\x0b \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0c \x01(\x05\x12\x18\n\x10variantMaxLength\x18\r \x01(\x05\x12\x0f\n\x07samples\x18\x0e \x03(\t\x12\x0c\n\x04skip\x18\x0f \x01(\x05\x12\r\n\x05limit\x18\x10 \x01(\x05\"\xf3\x02\n\x1f\x41llelesInRegionInSamplesRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0b\n\x03ref\x18\x04 \x01(\t\x12\x0b\n\x03\x61lt\x18\x05 \x01(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x0f\n\x07samples\x18\n \x03(\t\x12\x18\n\x10variantMinLength\x18\x0b \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0c \x01(\x05\x12\x0c\n\x04skip\x18\r \x01(\x05\x12\r\n\x05limit\x18\x0e \x01(\x05\"\xdf\x02\n\x1c\x41llelesInMultiRegionsRequest\x12\x31\n\x03\x63hr\x18\x01 \x03(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x03(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x03(\x05\x12\x0b\n\x03ref\x18\x04 \x03(\t\x12\x0b\n\x03\x61lt\x18\x05 \x03(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\x12\x0c\n\x04skip\x18\x0c \x01(\x05\x12\r\n\x05limit\x18\r \x01(\x05\"\xf9\x02\n%AllelesInMultiRegionsInSamplesRequest\x12\x31\n\x03\x63hr\x18\x01 \x03(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x03(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x03(\x05\x12\x0b\n\x03ref\x18\x04 \x03(\t\x12\x0b\n\x03\x61lt\x18\x05 \x03(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x0f\n\x07samples\x18\n \x03(\t\x12\x18\n\x10variantMinLength\x18\x0b \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0c \x01(\x05\x12\x0c\n\x04skip\x18\r \x01(\x05\x12\r\n\x05limit\x18\x0e \x01(\x05\"\xb0\x01\n\x0f\x41llelesResponse\x12\x33\n\x08variants\x18\x01 \x03(\x0b\x32!.org.dnaerys.cluster.grpc.Variant\x12\x1a\n\x12incomplete_cluster\x18\x02 \x01(\x08\x12\x10\n\x08\x61\x66\x66\x65\x63ted\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"\xef\x02\n\x07Variant\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0b\n\x03ref\x18\x04 \x01(\t\x12\x0b\n\x03\x61lt\x18\x05 \x01(\t\x12\n\n\x02\x61\x66\x18\x06 \x01(\x02\x12\n\n\x02\x61\x63\x18\x07 \x01(\x02\x12\n\n\x02\x61n\x18\x08 \x01(\x05\x12\x0c\n\x04homc\x18\t \x01(\x05\x12\x0c\n\x04hetc\x18\n \x01(\x05\x12\x0c\n\x04misc\x18\x0b \x01(\x05\x12\r\n\x05homfc\x18\x0c \x01(\x05\x12\r\n\x05hetfc\x18\r \x01(\x05\x12\r\n\x05misfc\x18\x0e \x01(\x05\x12\x0f\n\x07gnomADe\x18\x0f \x01(\x02\x12\x0f\n\x07gnomADg\x18\x10 \x01(\x02\x12\x10\n\x08\x63\x61\x64\x64_raw\x18\x11 \x01(\x02\x12\x12\n\ncadd_phred\x18\x12 \x01(\x02\x12\x10\n\x08\x61m_score\x18\x13 \x01(\x02\x12\x13\n\x0b\x61mino_acids\x18\x14 \x01(\t\x12\x11\n\tbiallelic\x18\x15 \x01(\x08\"\xc2\x01\n\x18\x41llelesWithStatsResponse\x12<\n\x08variants\x18\x01 \x03(\x0b\x32*.org.dnaerys.cluster.grpc.VariantWithStats\x12\x1a\n\x12incomplete_cluster\x18\x02 \x01(\x08\x12\x10\n\x08\x61\x66\x66\x65\x63ted\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"\xe1\x01\n\x10VariantWithStats\x12\x32\n\x07variant\x18\x01 \x01(\x0b\x32!.org.dnaerys.cluster.grpc.Variant\x12\x0b\n\x03vaf\x18\x02 \x01(\x02\x12\x0b\n\x03vac\x18\x03 \x01(\x02\x12\x0b\n\x03van\x18\x04 \x01(\x05\x12\r\n\x05vhomc\x18\x05 \x01(\x05\x12\r\n\x05vhetc\x18\x06 \x01(\x05\x12\x0e\n\x06vhomfc\x18\x07 \x01(\x05\x12\x0e\n\x06vhetfc\x18\x08 \x01(\x05\x12\x0c\n\x04phwe\x18\t \x01(\x02\x12\r\n\x05pchi2\x18\n \x01(\x02\x12\n\n\x02or\x18\x0b \x01(\x02\x12\x0b\n\x03ibc\x18\x0c \x01(\x02\":\n\x0fTopNchi2Request\x12\t\n\x01n\x18\x01 \x01(\x05\x12\x0f\n\x07samples\x18\x02 \x03(\t\x12\x0b\n\x03seq\x18\x03 \x01(\x08\"(\n\x0eTopNHWERequest\x12\t\n\x01n\x18\x01 \x01(\x05\x12\x0b\n\x03seq\x18\x02 \x01(\x08\"\xd9\x02\n\x16SamplesInRegionRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0b\n\x03ref\x18\x04 \x01(\t\x12\x0b\n\x03\x61lt\x18\x05 \x01(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0b \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0c \x01(\x05\x12\x0c\n\x04skip\x18\r \x01(\x05\x12\r\n\x05limit\x18\x0e \x01(\x05\"\xdf\x02\n\x1cSamplesInMultiRegionsRequest\x12\x31\n\x03\x63hr\x18\x01 \x03(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x03(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x03(\x05\x12\x0b\n\x03ref\x18\x04 \x03(\t\x12\x0b\n\x03\x61lt\x18\x05 \x03(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\x12\x0c\n\x04skip\x18\x0c \x01(\x05\x12\r\n\x05limit\x18\r \x01(\x05\"\xb1\x01\n\x14SamplesHomRefRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\x10\n\x08position\x18\x02 \x01(\x05\x12\x37\n\x08\x61ssembly\x18\x03 \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x0c\n\x04skip\x18\x04 \x01(\x05\x12\r\n\x05limit\x18\x05 \x01(\x05\"\x8c\x01\n\x0fSamplesResponse\x12\x0f\n\x07samples\x18\x01 \x03(\t\x12\x1a\n\x12incomplete_cluster\x18\x02 \x01(\x08\x12\x10\n\x08\x61\x66\x66\x65\x63ted\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"\xe9\x02\n\rDeNovoRequest\x12\x0f\n\x07parent1\x18\x01 \x01(\t\x12\x0f\n\x07parent2\x18\x02 \x01(\t\x12\x0f\n\x07proband\x18\x03 \x01(\t\x12\x31\n\x03\x63hr\x18\x04 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x05 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x06 \x01(\x05\x12\x0b\n\x03ref\x18\x07 \x01(\t\x12\x0b\n\x03\x61lt\x18\x08 \x01(\t\x12\x32\n\x03\x61nn\x18\t \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\n \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0b \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0c \x01(\x05\x12\x0c\n\x04skip\x18\r \x01(\x05\x12\r\n\x05limit\x18\x0e \x01(\x05\"\x87\x03\n\x12HetDominantRequest\x12\x17\n\x0f\x61\x66\x66\x65\x63ted_parent\x18\x01 \x01(\t\x12\x19\n\x11unaffected_parent\x18\x02 \x01(\t\x12\x16\n\x0e\x61\x66\x66\x65\x63ted_child\x18\x03 \x01(\t\x12\x31\n\x03\x63hr\x18\x04 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x05 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x06 \x01(\x05\x12\x0b\n\x03ref\x18\x07 \x01(\t\x12\x0b\n\x03\x61lt\x18\x08 \x01(\t\x12\x32\n\x03\x61nn\x18\t \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\n \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0b \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0c \x01(\x05\x12\x0c\n\x04skip\x18\r \x01(\x05\x12\r\n\x05limit\x18\x0e \x01(\x05\"\x8c\x03\n\x13HomRecessiveRequest\x12\x1a\n\x12unaffected_parent1\x18\x01 \x01(\t\x12\x1a\n\x12unaffected_parent2\x18\x02 \x01(\t\x12\x16\n\x0e\x61\x66\x66\x65\x63ted_child\x18\x03 \x01(\t\x12\x31\n\x03\x63hr\x18\x04 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x05 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x06 \x01(\x05\x12\x0b\n\x03ref\x18\x07 \x01(\t\x12\x0b\n\x03\x61lt\x18\x08 \x01(\t\x12\x32\n\x03\x61nn\x18\t \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\n \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0b \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0c \x01(\x05\x12\x0c\n\x04skip\x18\r \x01(\x05\x12\r\n\x05limit\x18\x0e \x01(\x05\"i\n\nPRSRequest\x12\x10\n\x08prs_name\x18\x01 \x01(\t\x12\x13\n\x0b\x63ohort_name\x18\x02 \x01(\t\x12\x0f\n\x07samples\x18\x03 \x03(\t\x12\x10\n\x08\x64ominant\x18\x04 \x01(\x08\x12\x11\n\trecessive\x18\x05 \x01(\x08\"\xf3\x01\n\x0bPRSResponse\x12\x10\n\x08prs_name\x18\x01 \x01(\t\x12<\n\rsample_scores\x18\x02 \x03(\x0b\x32%.org.dnaerys.cluster.grpc.SampleScore\x12\x10\n\x08\x64ominant\x18\x03 \x01(\x08\x12\x11\n\trecessive\x18\x04 \x01(\x08\x12\x17\n\x0fprs_cardinality\x18\x05 \x01(\x05\x12\x1a\n\x12incomplete_cluster\x18\x06 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x07 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x08 \x01(\x03\x12\x0f\n\x07node_id\x18\t \x01(\t\"\x94\x01\n\x0bSampleScore\x12\x0e\n\x06sample\x18\x01 \x01(\t\x12\x12\n\nscores_sum\x18\x02 \x01(\x02\x12\x1a\n\x12hethom_cardinality\x18\x03 \x01(\x05\x12\x17\n\x0fref_cardinality\x18\x04 \x01(\x05\x12\x17\n\x0fmis_cardinality\x18\x05 \x01(\x05\x12\x13\n\x0bimputed_sum\x18\x06 \x01(\x02\"\xa0\x01\n\rFstatXRequest\x12\x13\n\x0b\x63ohort_name\x18\x01 \x01(\t\x12\x0f\n\x07samples\x18\x02 \x03(\t\x12\x15\n\raaf_threshold\x18\x03 \x01(\x02\x12\x18\n\x10\x66\x65male_threshold\x18\x04 \x01(\x02\x12\x16\n\x0emale_threshold\x18\x05 \x01(\x02\x12\x13\n\x0binclude_par\x18\x06 \x01(\x08\x12\x0b\n\x03seq\x18\x07 \x01(\x08\"\xeb\x01\n\x13SexMismatchResponse\x12<\n\x0emismatch_males\x18\x01 \x03(\x0b\x32$.org.dnaerys.cluster.grpc.SampleStat\x12>\n\x10mismatch_females\x18\x02 \x03(\x0b\x32$.org.dnaerys.cluster.grpc.SampleStat\x12\x1a\n\x12incomplete_cluster\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"X\n\nSampleStat\x12\x0e\n\x06sample\x18\x01 \x01(\t\x12\x14\n\x0creported_sex\x18\x02 \x01(\t\x12\x14\n\x0cobserved_sex\x18\x03 \x01(\t\x12\x0e\n\x06\x66_stat\x18\x04 \x01(\x02\"\xd4\x01\n\x0e\x46statXResponse\x12\x33\n\x05males\x18\x01 \x03(\x0b\x32$.org.dnaerys.cluster.grpc.SampleStat\x12\x35\n\x07\x66\x65males\x18\x02 \x03(\x0b\x32$.org.dnaerys.cluster.grpc.SampleStat\x12\x1a\n\x12incomplete_cluster\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"\x8f\x01\n\x0eKinshipRequest\x12\x13\n\x0b\x63ohort_name\x18\x01 \x01(\t\x12\x0f\n\x07samples\x18\x02 \x03(\t\x12\x37\n\x06\x64\x65gree\x18\x03 \x01(\x0e\x32\'.org.dnaerys.cluster.grpc.KinshipDegree\x12\x11\n\tthreshold\x18\x04 \x01(\x02\x12\x0b\n\x03seq\x18\x05 \x01(\x08\"B\n\x11KinshipDuoRequest\x12\x0f\n\x07sample1\x18\x01 \x01(\t\x12\x0f\n\x07sample2\x18\x02 \x01(\t\x12\x0b\n\x03seq\x18\x03 \x01(\x08\"\xa0\x01\n\x12KinshipTrioRequest\x12\x0f\n\x07sample1\x18\x01 \x01(\t\x12\x0f\n\x07sample2\x18\x02 \x01(\t\x12\x0f\n\x07sample3\x18\x03 \x01(\t\x12\x37\n\x06\x64\x65gree\x18\x04 \x01(\x0e\x32\'.org.dnaerys.cluster.grpc.KinshipDegree\x12\x11\n\tthreshold\x18\x05 \x01(\x02\x12\x0b\n\x03seq\x18\x06 \x01(\x08\"\x9d\x01\n\x0fKinshipResponse\x12\x32\n\x03rel\x18\x01 \x03(\x0b\x32%.org.dnaerys.cluster.grpc.Relatedness\x12\x1a\n\x12incomplete_cluster\x18\x02 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x03 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x04 \x01(\x03\x12\x0f\n\x07node_id\x18\x05 \x01(\t\"y\n\x0bRelatedness\x12\x0f\n\x07sample1\x18\x01 \x01(\t\x12\x0f\n\x07sample2\x18\x02 \x01(\t\x12\x37\n\x06\x64\x65gree\x18\x03 \x01(\x0e\x32\'.org.dnaerys.cluster.grpc.KinshipDegree\x12\x0f\n\x07phi_bwf\x18\x04 \x01(\x02\"\x98\x01\n\x14SampleKinshipRequest\x12\x12\n\nsample_vcf\x18\x01 \x01(\t\x12\x13\n\x0b\x63ohort_name\x18\x02 \x01(\t\x12\x37\n\x06\x64\x65gree\x18\x03 \x01(\x0e\x32\'.org.dnaerys.cluster.grpc.KinshipDegree\x12\x11\n\tthreshold\x18\x04 \x01(\x02\x12\x0b\n\x03seq\x18\x05 \x01(\x08\"\xc3\x01\n\x15SampleKinshipResponse\x12;\n\x03rel\x18\x01 \x03(\x0b\x32..org.dnaerys.cluster.grpc.RelatednessPerSample\x12\x15\n\raccepted_snvs\x18\x02 \x01(\x05\x12\x1a\n\x12incomplete_cluster\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"\xc7\x01\n\x14RelatednessPerSample\x12\x0e\n\x06sample\x18\x01 \x01(\t\x12\x37\n\x06\x64\x65gree\x18\x02 \x01(\x0e\x32\'.org.dnaerys.cluster.grpc.KinshipDegree\x12\x0f\n\x07phi_bwf\x18\x03 \x01(\x02\x12\x13\n\x0b\x63ommon_loci\x18\x04 \x01(\x05\x12\x0e\n\x06nHetS1\x18\x05 \x01(\x05\x12\x0e\n\x06nHetS2\x18\x06 \x01(\x05\x12\x10\n\x08nHetS1S2\x18\x07 \x01(\x05\x12\x0e\n\x06nHomOp\x18\x08 \x01(\x05\"\xc1\x02\n\x1b\x43ountAllelesInRegionRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0b\n\x03ref\x18\x04 \x01(\t\x12\x0b\n\x03\x61lt\x18\x05 \x01(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\"\xdb\x02\n$CountAllelesInRegionInSamplesRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0b\n\x03ref\x18\x04 \x01(\t\x12\x0b\n\x03\x61lt\x18\x05 \x01(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\x12\x0f\n\x07samples\x18\x0c \x03(\t\"\xee\x02\n\x1c\x43ountAllelesInBracketRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\x11\n\tstart_min\x18\x02 \x01(\x05\x12\x11\n\tstart_max\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nd_min\x18\x04 \x01(\x05\x12\x0f\n\x07\x65nd_max\x18\x05 \x01(\x05\x12\x0b\n\x03ref\x18\x06 \x01(\t\x12\x0b\n\x03\x61lt\x18\x07 \x01(\t\x12\x0b\n\x03hom\x18\x08 \x01(\x08\x12\x0b\n\x03het\x18\t \x01(\x08\x12\x32\n\x03\x61nn\x18\n \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\x0b \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0c \x01(\x05\x12\x18\n\x10variantMaxLength\x18\r \x01(\x05\"\x88\x03\n%CountAllelesInBracketInSamplesRequest\x12\x31\n\x03\x63hr\x18\x01 \x01(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\x11\n\tstart_min\x18\x02 \x01(\x05\x12\x11\n\tstart_max\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nd_min\x18\x04 \x01(\x05\x12\x0f\n\x07\x65nd_max\x18\x05 \x01(\x05\x12\x0b\n\x03ref\x18\x06 \x01(\t\x12\x0b\n\x03\x61lt\x18\x07 \x01(\t\x12\x0b\n\x03hom\x18\x08 \x01(\x08\x12\x0b\n\x03het\x18\t \x01(\x08\x12\x32\n\x03\x61nn\x18\n \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\x0b \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\x0c \x01(\x05\x12\x18\n\x10variantMaxLength\x18\r \x01(\x05\x12\x0f\n\x07samples\x18\x0e \x03(\t\"\xc7\x02\n!CountAllelesInMultiRegionsRequest\x12\x31\n\x03\x63hr\x18\x01 \x03(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x03(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x03(\x05\x12\x0b\n\x03ref\x18\x04 \x03(\t\x12\x0b\n\x03\x61lt\x18\x05 \x03(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\"\xe1\x02\n*CountAllelesInMultiRegionsInSamplesRequest\x12\x31\n\x03\x63hr\x18\x01 \x03(\x0e\x32$.org.dnaerys.cluster.grpc.Chromosome\x12\r\n\x05start\x18\x02 \x03(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x03(\x05\x12\x0b\n\x03ref\x18\x04 \x03(\t\x12\x0b\n\x03\x61lt\x18\x05 \x03(\t\x12\x0b\n\x03hom\x18\x06 \x01(\x08\x12\x0b\n\x03het\x18\x07 \x01(\x08\x12\x32\n\x03\x61nn\x18\x08 \x01(\x0b\x32%.org.dnaerys.cluster.grpc.Annotations\x12\x37\n\x08\x61ssembly\x18\t \x01(\x0e\x32%.org.dnaerys.cluster.grpc.RefAssembly\x12\x18\n\x10variantMinLength\x18\n \x01(\x05\x12\x18\n\x10variantMaxLength\x18\x0b \x01(\x05\x12\x0f\n\x07samples\x18\x0c \x03(\t\"\x8f\x01\n\x14\x43ountAllelesResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12\x1a\n\x12incomplete_cluster\x18\x02 \x01(\x08\x12\x10\n\x08\x61\x66\x66\x65\x63ted\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t\"\x8f\x01\n\x14\x43ountSamplesResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\x12\x1a\n\x12incomplete_cluster\x18\x02 \x01(\x08\x12\x10\n\x08\x61\x66\x66\x65\x63ted\x18\x03 \x01(\x08\x12\x12\n\nelapsed_ms\x18\x04 \x01(\x03\x12\x15\n\relapsed_db_ms\x18\x05 \x01(\x03\x12\x0f\n\x07node_id\x18\x06 \x01(\t*?\n\x0bRefAssembly\x12\x18\n\x14\x41SSEMBLY_UNSPECIFIED\x10\x00\x12\n\n\x06GRCh37\x10\x01\x12\n\n\x06GRCh38\x10\x02*\xc9\x02\n\nChromosome\x12\x1a\n\x16\x43HROMOSOME_UNSPECIFIED\x10\x00\x12\t\n\x05\x43HR_1\x10\x01\x12\t\n\x05\x43HR_2\x10\x02\x12\t\n\x05\x43HR_3\x10\x03\x12\t\n\x05\x43HR_4\x10\x04\x12\t\n\x05\x43HR_5\x10\x05\x12\t\n\x05\x43HR_6\x10\x06\x12\t\n\x05\x43HR_7\x10\x07\x12\t\n\x05\x43HR_8\x10\x08\x12\t\n\x05\x43HR_9\x10\t\x12\n\n\x06\x43HR_10\x10\n\x12\n\n\x06\x43HR_11\x10\x0b\x12\n\n\x06\x43HR_12\x10\x0c\x12\n\n\x06\x43HR_13\x10\r\x12\n\n\x06\x43HR_14\x10\x0e\x12\n\n\x06\x43HR_15\x10\x0f\x12\n\n\x06\x43HR_16\x10\x10\x12\n\n\x06\x43HR_17\x10\x11\x12\n\n\x06\x43HR_18\x10\x12\x12\n\n\x06\x43HR_19\x10\x13\x12\n\n\x06\x43HR_20\x10\x14\x12\n\n\x06\x43HR_21\x10\x15\x12\n\n\x06\x43HR_22\x10\x16\x12\t\n\x05\x43HR_X\x10\x17\x12\t\n\x05\x43HR_Y\x10\x18\x12\n\n\x06\x43HR_MT\x10\x19*\xc3\x06\n\x0bVariantType\x12\x1b\n\x17VARIANTTYPE_UNSPECIFIED\x10\x00\x12\x07\n\x03SNV\x10\x01\x12\r\n\tINSERTION\x10\x02\x12\x0c\n\x08\x44\x45LETION\x10\x03\x12\t\n\x05INDEL\x10\x04\x12\x10\n\x0cSUBSTITUTION\x10\x05\x12\r\n\tINVERSION\x10\x06\x12\x11\n\rTRANSLOCATION\x10\x07\x12\x0f\n\x0b\x44UPLICATION\x10\x08\x12\x11\n\rALU_INSERTION\x10\t\x12!\n\x1d\x43OMPLEX_STRUCTURAL_ALTERATION\x10\n\x12\x18\n\x14\x43OMPLEX_SUBSTITUTION\x10\x0b\x12\x14\n\x10\x43OPY_NUMBER_GAIN\x10\x0c\x12\x14\n\x10\x43OPY_NUMBER_LOSS\x10\r\x12\x19\n\x15\x43OPY_NUMBER_VARIATION\x10\x0e\x12\x1f\n\x1bINTERCHROMOSOMAL_BREAKPOINT\x10\x0f\x12\"\n\x1eINTERCHROMOSOMAL_TRANSLOCATION\x10\x10\x12\x1f\n\x1bINTRACHROMOSOMAL_BREAKPOINT\x10\x11\x12\"\n\x1eINTRACHROMOSOMAL_TRANSLOCATION\x10\x12\x12\x1a\n\x16LOSS_OF_HETEROZYGOSITY\x10\x13\x12\x1b\n\x17MOBILE_ELEMENT_DELETION\x10\x14\x12\x1c\n\x18MOBILE_ELEMENT_INSERTION\x10\x15\x12\x1c\n\x18NOVEL_SEQUENCE_INSERTION\x10\x16\x12!\n\x1dSHORT_TANDEM_REPEAT_VARIATION\x10\x17\x12\x16\n\x12TANDEM_DUPLICATION\x10\x18\x12\t\n\x05PROBE\x10\x19\x12\x10\n\x0c\x41LU_DELETION\x10\x1a\x12\x11\n\rHERV_DELETION\x10\x1b\x12\x12\n\x0eHERV_INSERTION\x10\x1c\x12\x12\n\x0eLINE1_DELETION\x10\x1d\x12\x13\n\x0fLINE1_INSERTION\x10\x1e\x12\x10\n\x0cSVA_DELETION\x10\x1f\x12\x11\n\rSVA_INSERTION\x10 \x12%\n!COMPLEX_CHROMOSOMAL_REARRANGEMENT\x10!\x12\x17\n\x13SEQUENCE_ALTERATION\x10\"*c\n\x0b\x46\x65\x61tureType\x12\x1b\n\x17\x46\x45\x41TURETYPE_UNSPECIFIED\x10\x00\x12\x0e\n\nTRANSCRIPT\x10\x01\x12\x15\n\x11REGULATORYFEATURE\x10\x02\x12\x10\n\x0cMOTIFFEATURE\x10\x03*\xe1\x06\n\x07\x42ioType\x12\x17\n\x13\x42IOTYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14PROCESSED_TRANSCRIPT\x10\x01\x12\n\n\x06LNCRNA\x10\x02\x12\r\n\tANTISENSE\x10\x03\x12\x10\n\x0cMACRO_LNCRNA\x10\x04\x12\x0e\n\nNON_CODING\x10\x05\x12\x13\n\x0fRETAINED_INTRON\x10\x06\x12\x12\n\x0eSENSE_INTRONIC\x10\x07\x12\x15\n\x11SENSE_OVERLAPPING\x10\x08\x12\x0b\n\x07LINCRNA\x10\t\x12\t\n\x05NCRNA\x10\n\x12\t\n\x05MIRNA\x10\x0b\x12\x0b\n\x07MISCRNA\x10\x0c\x12\t\n\x05PIRNA\x10\r\x12\x08\n\x04RRNA\x10\x0e\x12\t\n\x05SIRNA\x10\x0f\x12\t\n\x05SNRNA\x10\x10\x12\n\n\x06SNORNA\x10\x11\x12\x08\n\x04TRNA\x10\x12\x12\x0c\n\x08VAULTRNA\x10\x13\x12\x12\n\x0ePROTEIN_CODING\x10\x14\x12\x0e\n\nPSEUDOGENE\x10\x15\x12\x11\n\rIG_PSEUDOGENE\x10\x16\x12\x1a\n\x16POLYMORPHIC_PSEUDOGENE\x10\x17\x12\x18\n\x14PROCESSED_PSEUDOGENE\x10\x18\x12\x1a\n\x16TRANSCRIBED_PSEUDOGENE\x10\x19\x12\x19\n\x15TRANSLATED_PSEUDOGENE\x10\x1a\x12\x16\n\x12UNITARY_PSEUDOGENE\x10\x1b\x12\x1a\n\x16UNPROCESSED_PSEUDOGENE\x10\x1c\x12\x0f\n\x0bREADTHROUGH\x10\x1d\x12\x1a\n\x16STOP_CODON_READTHROUGH\x10\x1e\x12\x07\n\x03TEC\x10\x1f\x12\x0b\n\x07TR_GENE\x10 \x12\r\n\tTR_C_GENE\x10!\x12\r\n\tTR_D_GENE\x10\"\x12\r\n\tTR_J_GENE\x10#\x12\r\n\tTR_V_GENE\x10$\x12\x0b\n\x07IG_GENE\x10%\x12\r\n\tIG_C_GENE\x10&\x12\r\n\tIG_D_GENE\x10\'\x12\r\n\tIG_J_GENE\x10(\x12\r\n\tIG_V_GENE\x10)\x12\x1b\n\x17NONSENSE_MEDIATED_DECAY\x10*\x12\x0c\n\x08PROMOTER\x10+\x12\x1c\n\x18PROMOTER_FLANKING_REGION\x10,\x12\x0c\n\x08\x45NHANCER\x10-\x12\x15\n\x11\x43TCF_BINDING_SITE\x10.\x12\x19\n\x15OPEN_CHROMATIN_REGION\x10/*\x8b\t\n\x0b\x43onsequence\x12\x1b\n\x17\x43ONSEQUENCE_UNSPECIFIED\x10\x00\x12\x17\n\x13TRANSCRIPT_ABLATION\x10\x01\x12\x1b\n\x17SPLICE_ACCEPTOR_VARIANT\x10\x02\x12\x18\n\x14SPLICE_DONOR_VARIANT\x10\x03\x12\x0f\n\x0bSTOP_GAINED\x10\x04\x12\x16\n\x12\x46RAMESHIFT_VARIANT\x10\x05\x12\r\n\tSTOP_LOST\x10\x06\x12\x0e\n\nSTART_LOST\x10\x07\x12\x1c\n\x18TRANSCRIPT_AMPLIFICATION\x10\x08\x12\x15\n\x11INFRAME_INSERTION\x10\t\x12\x14\n\x10INFRAME_DELETION\x10\n\x12\x14\n\x10MISSENSE_VARIANT\x10\x0b\x12\x1c\n\x18PROTEIN_ALTERING_VARIANT\x10\x0c\x12\x19\n\x15SPLICE_REGION_VARIANT\x10\r\x12%\n!INCOMPLETE_TERMINAL_CODON_VARIANT\x10\x0e\x12\x1a\n\x16START_RETAINED_VARIANT\x10\x0f\x12\x19\n\x15STOP_RETAINED_VARIANT\x10\x10\x12\x16\n\x12SYNONYMOUS_VARIANT\x10\x11\x12\x1b\n\x17\x43ODING_SEQUENCE_VARIANT\x10\x12\x12\x18\n\x14MATURE_MIRNA_VARIANT\x10\x13\x12\x1a\n\x16\x46IVE_PRIME_UTR_VARIANT\x10\x14\x12\x1b\n\x17THREE_PRIME_UTR_VARIANT\x10\x15\x12&\n\"NON_CODING_TRANSCRIPT_EXON_VARIANT\x10\x16\x12\x12\n\x0eINTRON_VARIANT\x10\x17\x12\x1a\n\x16NMD_TRANSCRIPT_VARIANT\x10\x18\x12!\n\x1dNON_CODING_TRANSCRIPT_VARIANT\x10\x19\x12\x19\n\x15UPSTREAM_GENE_VARIANT\x10\x1a\x12\x1b\n\x17\x44OWNSTREAM_GENE_VARIANT\x10\x1b\x12\x11\n\rTFBS_ABLATION\x10\x1c\x12\x16\n\x12TFBS_AMPLIFICATION\x10\x1d\x12\x1b\n\x17TF_BINDING_SITE_VARIANT\x10\x1e\x12\x1e\n\x1aREGULATORY_REGION_ABLATION\x10\x1f\x12#\n\x1fREGULATORY_REGION_AMPLIFICATION\x10 \x12\x16\n\x12\x46\x45\x41TURE_ELONGATION\x10!\x12\x1d\n\x19REGULATORY_REGION_VARIANT\x10\"\x12\x16\n\x12\x46\x45\x41TURE_TRUNCATION\x10#\x12\x16\n\x12INTERGENIC_VARIANT\x10$\x12\'\n#SPLICE_POLYPYRIMIDINE_TRACT_VARIANT\x10%\x12!\n\x1dSPLICE_DONOR_5TH_BASE_VARIANT\x10&\x12\x1f\n\x1bSPLICE_DONOR_REGION_VARIANT\x10\'\x12\x1d\n\x19\x43ODING_TRANSCRIPT_VARIANT\x10(\x12\x14\n\x10SEQUENCE_VARIANT\x10)*O\n\x06Impact\x12\x16\n\x12IMPACT_UNSPECIFIED\x10\x00\x12\x08\n\x04HIGH\x10\x01\x12\x0c\n\x08MODERATE\x10\x02\x12\x07\n\x03LOW\x10\x03\x12\x0c\n\x08MODIFIER\x10\x04*<\n\x04SIFT\x12\x14\n\x10SIFT_UNSPECIFIED\x10\x00\x12\r\n\tTOLERATED\x10\x01\x12\x0f\n\x0b\x44\x45LETERIOUS\x10\x02*k\n\x08PolyPhen\x12\x18\n\x14POLYPHEN_UNSPECIFIED\x10\x00\x12\n\n\x06\x42\x45NIGN\x10\x01\x12\x15\n\x11POSSIBLY_DAMAGING\x10\x02\x12\x15\n\x11PROBABLY_DAMAGING\x10\x03\x12\x0b\n\x07UNKNOWN\x10\x04*\xd1\x03\n\x10\x43linSignificance\x12\x16\n\x12\x43LNSIG_UNSPECIFIED\x10\x00\x12\x11\n\rCLNSIG_BENIGN\x10\x01\x12\x11\n\rLIKELY_BENIGN\x10\x02\x12\x1a\n\x16UNCERTAIN_SIGNIFICANCE\x10\x03\x12\x15\n\x11LIKELY_PATHOGENIC\x10\x04\x12\x0e\n\nPATHOGENIC\x10\x05\x12\x11\n\rDRUG_RESPONSE\x10\x06\x12\x0f\n\x0b\x41SSOCIATION\x10\x07\x12\x0f\n\x0bRISK_FACTOR\x10\x08\x12\x0e\n\nPROTECTIVE\x10\t\x12\x0b\n\x07\x41\x46\x46\x45\x43TS\x10\n\x12\x17\n\x13\x43ONFERS_SENSITIVITY\x10\x0b\x12\x1f\n\x1b\x43ONFLICTING_INTERPRETATIONS\x10\x0c\x12\x10\n\x0cNOT_PROVIDED\x10\r\x12\t\n\x05OTHER\x10\x0e\x12$\n LIKELY_PATHOGENIC_LOW_PENETRANCE\x10\x0f\x12\x1d\n\x19PATHOGENIC_LOW_PENETRANCE\x10\x10\x12\x19\n\x15UNCERTAIN_RISK_ALLELE\x10\x11\x12\x16\n\x12LIKELY_RISK_ALLELE\x10\x12\x12\x1b\n\x17\x45STABLISHED_RISK_ALLELE\x10\x13*e\n\rAlphaMissense\x12\x12\n\x0e\x41M_UNSPECIFIED\x10\x00\x12\x14\n\x10\x41M_LIKELY_BENIGN\x10\x01\x12\x18\n\x14\x41M_LIKELY_PATHOGENIC\x10\x02\x12\x10\n\x0c\x41M_AMBIGUOUS\x10\x03*\x85\x01\n\rKinshipDegree\x12\x17\n\x13KINSHIP_UNSPECIFIED\x10\x00\x12\x15\n\x11TWINS_MONOZYGOTIC\x10\x01\x12\x10\n\x0c\x46IRST_DEGREE\x10\x02\x12\x11\n\rSECOND_DEGREE\x10\x03\x12\x10\n\x0cTHIRD_DEGREE\x10\x04\x12\r\n\tUNRELATED\x10\x05\x32\xbb#\n\x0e\x44naerysService\x12]\n\x06Health\x12\'.org.dnaerys.cluster.grpc.HealthRequest\x1a(.org.dnaerys.cluster.grpc.HealthResponse\"\x00\x12o\n\x0c\x43lusterNodes\x12-.org.dnaerys.cluster.grpc.ClusterNodesRequest\x1a..org.dnaerys.cluster.grpc.ClusterNodesResponse\"\x00\x12l\n\x0b\x44\x61tasetInfo\x12,.org.dnaerys.cluster.grpc.DatasetInfoRequest\x1a-.org.dnaerys.cluster.grpc.DatasetInfoResponse\"\x00\x12y\n\x16SelectVariantsInRegion\x12\x30.org.dnaerys.cluster.grpc.AllelesInRegionRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12\x80\x01\n\x15\x43ountVariantsInRegion\x12\x35.org.dnaerys.cluster.grpc.CountAllelesInRegionRequest\x1a..org.dnaerys.cluster.grpc.CountAllelesResponse\"\x00\x12\x8b\x01\n\x1fSelectVariantsInRegionInSamples\x12\x39.org.dnaerys.cluster.grpc.AllelesInRegionInSamplesRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12\x92\x01\n\x1e\x43ountVariantsInRegionInSamples\x12>.org.dnaerys.cluster.grpc.CountAllelesInRegionInSamplesRequest\x1a..org.dnaerys.cluster.grpc.CountAllelesResponse\"\x00\x12\x9d\x01\n(SelectVariantsInRegionInSamplesWithStats\x12\x39.org.dnaerys.cluster.grpc.AllelesInRegionInSamplesRequest\x1a\x32.org.dnaerys.cluster.grpc.AllelesWithStatsResponse\"\x00\x30\x01\x12{\n\x17SelectVariantsInBracket\x12\x31.org.dnaerys.cluster.grpc.AllelesInBracketRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12\x82\x01\n\x16\x43ountVariantsInBracket\x12\x36.org.dnaerys.cluster.grpc.CountAllelesInBracketRequest\x1a..org.dnaerys.cluster.grpc.CountAllelesResponse\"\x00\x12\x8d\x01\n SelectVariantsInBracketInSamples\x12:.org.dnaerys.cluster.grpc.AllelesInBracketInSamplesRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12\x94\x01\n\x1f\x43ountVariantsInBracketInSamples\x12?.org.dnaerys.cluster.grpc.CountAllelesInBracketInSamplesRequest\x1a..org.dnaerys.cluster.grpc.CountAllelesResponse\"\x00\x12\x85\x01\n\x1cSelectVariantsInMultiRegions\x12\x36.org.dnaerys.cluster.grpc.AllelesInMultiRegionsRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12\x8c\x01\n\x1b\x43ountVariantsInMultiRegions\x12;.org.dnaerys.cluster.grpc.CountAllelesInMultiRegionsRequest\x1a..org.dnaerys.cluster.grpc.CountAllelesResponse\"\x00\x12\x97\x01\n%SelectVariantsInMultiRegionsWithStats\x12\x36.org.dnaerys.cluster.grpc.AllelesInMultiRegionsRequest\x1a\x32.org.dnaerys.cluster.grpc.AllelesWithStatsResponse\"\x00\x30\x01\x12\x97\x01\n%SelectVariantsInMultiRegionsInSamples\x12?.org.dnaerys.cluster.grpc.AllelesInMultiRegionsInSamplesRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12\x9e\x01\n$CountVariantsInMultiRegionsInSamples\x12\x44.org.dnaerys.cluster.grpc.CountAllelesInMultiRegionsInSamplesRequest\x1a..org.dnaerys.cluster.grpc.CountAllelesResponse\"\x00\x12\xa9\x01\n.SelectVariantsInMultiRegionsInSamplesWithStats\x12?.org.dnaerys.cluster.grpc.AllelesInMultiRegionsInSamplesRequest\x1a\x32.org.dnaerys.cluster.grpc.AllelesWithStatsResponse\"\x00\x30\x01\x12i\n\x07TopNHWE\x12(.org.dnaerys.cluster.grpc.TopNHWERequest\x1a\x32.org.dnaerys.cluster.grpc.AllelesWithStatsResponse\"\x00\x12k\n\x08TopNchi2\x12).org.dnaerys.cluster.grpc.TopNchi2Request\x1a\x32.org.dnaerys.cluster.grpc.AllelesWithStatsResponse\"\x00\x12v\n\x15SelectSamplesInRegion\x12\x30.org.dnaerys.cluster.grpc.SamplesInRegionRequest\x1a).org.dnaerys.cluster.grpc.SamplesResponse\"\x00\x12z\n\x14\x43ountSamplesInRegion\x12\x30.org.dnaerys.cluster.grpc.SamplesInRegionRequest\x1a..org.dnaerys.cluster.grpc.CountSamplesResponse\"\x00\x12\x82\x01\n\x1bSelectSamplesInMultiRegions\x12\x36.org.dnaerys.cluster.grpc.SamplesInMultiRegionsRequest\x1a).org.dnaerys.cluster.grpc.SamplesResponse\"\x00\x12\x86\x01\n\x1a\x43ountSamplesInMultiRegions\x12\x36.org.dnaerys.cluster.grpc.SamplesInMultiRegionsRequest\x1a..org.dnaerys.cluster.grpc.CountSamplesResponse\"\x00\x12x\n\x19SelectSamplesHomReference\x12..org.dnaerys.cluster.grpc.SamplesHomRefRequest\x1a).org.dnaerys.cluster.grpc.SamplesResponse\"\x00\x12|\n\x18\x43ountSamplesHomReference\x12..org.dnaerys.cluster.grpc.SamplesHomRefRequest\x1a..org.dnaerys.cluster.grpc.CountSamplesResponse\"\x00\x12\x66\n\x0cSelectDeNovo\x12\'.org.dnaerys.cluster.grpc.DeNovoRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12p\n\x11SelectHetDominant\x12,.org.dnaerys.cluster.grpc.HetDominantRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12r\n\x12SelectHomRecessive\x12-.org.dnaerys.cluster.grpc.HomRecessiveRequest\x1a).org.dnaerys.cluster.grpc.AllelesResponse\"\x00\x30\x01\x12T\n\x03Prs\x12$.org.dnaerys.cluster.grpc.PRSRequest\x1a%.org.dnaerys.cluster.grpc.PRSResponse\"\x00\x12l\n\x10SexMismatchCheck\x12\'.org.dnaerys.cluster.grpc.FstatXRequest\x1a-.org.dnaerys.cluster.grpc.SexMismatchResponse\"\x00\x12]\n\x06\x46statX\x12\'.org.dnaerys.cluster.grpc.FstatXRequest\x1a(.org.dnaerys.cluster.grpc.FstatXResponse\"\x00\x12`\n\x07Kinship\x12(.org.dnaerys.cluster.grpc.KinshipRequest\x1a).org.dnaerys.cluster.grpc.KinshipResponse\"\x00\x12\x66\n\nKinshipDuo\x12+.org.dnaerys.cluster.grpc.KinshipDuoRequest\x1a).org.dnaerys.cluster.grpc.KinshipResponse\"\x00\x12h\n\x0bKinshipTrio\x12,.org.dnaerys.cluster.grpc.KinshipTrioRequest\x1a).org.dnaerys.cluster.grpc.KinshipResponse\"\x00\x12r\n\rSampleKinship\x12..org.dnaerys.cluster.grpc.SampleKinshipRequest\x1a/.org.dnaerys.cluster.grpc.SampleKinshipResponse\"\x00\x42\x10\x42\x0c\x44naerysProtoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dnaerys_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\014DnaerysProtoP\001'
  _globals['_REFASSEMBLY']._serialized_start=12164
  _globals['_REFASSEMBLY']._serialized_end=12227
  _globals['_CHROMOSOME']._serialized_start=12230
  _globals['_CHROMOSOME']._serialized_end=12559
  _globals['_VARIANTTYPE']._serialized_start=12562
  _globals['_VARIANTTYPE']._serialized_end=13397
  _globals['_FEATURETYPE']._serialized_start=13399
  _globals['_FEATURETYPE']._serialized_end=13498
  _globals['_BIOTYPE']._serialized_start=13501
  _globals['_BIOTYPE']._serialized_end=14366
  _globals['_CONSEQUENCE']._serialized_start=14369
  _globals['_CONSEQUENCE']._serialized_end=15532
  _globals['_IMPACT']._serialized_start=15534
  _globals['_IMPACT']._serialized_end=15613
  _globals['_SIFT']._serialized_start=15615
  _globals['_SIFT']._serialized_end=15675
  _globals['_POLYPHEN']._serialized_start=15677
  _globals['_POLYPHEN']._serialized_end=15784
  _globals['_CLINSIGNIFICANCE']._serialized_start=15787
  _globals['_CLINSIGNIFICANCE']._serialized_end=16252
  _globals['_ALPHAMISSENSE']._serialized_start=16254
  _globals['_ALPHAMISSENSE']._serialized_end=16355
  _globals['_KINSHIPDEGREE']._serialized_start=16358
  _globals['_KINSHIPDEGREE']._serialized_end=16491
  _globals['_HEALTHREQUEST']._serialized_start=43
  _globals['_HEALTHREQUEST']._serialized_end=58
  _globals['_HEALTHRESPONSE']._serialized_start=60
  _globals['_HEALTHRESPONSE']._serialized_end=92
  _globals['_CLUSTERNODESREQUEST']._serialized_start=94
  _globals['_CLUSTERNODESREQUEST']._serialized_end=115
  _globals['_CLUSTERNODESRESPONSE']._serialized_start=117
  _globals['_CLUSTERNODESRESPONSE']._serialized_end=226
  _globals['_DATASETINFOREQUEST']._serialized_start=228
  _globals['_DATASETINFOREQUEST']._serialized_end=278
  _globals['_DATASETINFORESPONSE']._serialized_start=281
  _globals['_DATASETINFORESPONSE']._serialized_end=671
  _globals['_COHORT']._serialized_start=674
  _globals['_COHORT']._serialized_end=845
  _globals['_PRS']._serialized_start=847
  _globals['_PRS']._serialized_end=901
  _globals['_ANNOTATIONS']._serialized_start=904
  _globals['_ANNOTATIONS']._serialized_end=1795
  _globals['_ALLELESINREGIONREQUEST']._serialized_start=1798
  _globals['_ALLELESINREGIONREQUEST']._serialized_end=2143
  _globals['_ALLELESINBRACKETREQUEST']._serialized_start=2146
  _globals['_ALLELESINBRACKETREQUEST']._serialized_end=2536
  _globals['_ALLELESINBRACKETINSAMPLESREQUEST']._serialized_start=2539
  _globals['_ALLELESINBRACKETINSAMPLESREQUEST']._serialized_end=2955
  _globals['_ALLELESINREGIONINSAMPLESREQUEST']._serialized_start=2958
  _globals['_ALLELESINREGIONINSAMPLESREQUEST']._serialized_end=3329
  _globals['_ALLELESINMULTIREGIONSREQUEST']._serialized_start=3332
  _globals['_ALLELESINMULTIREGIONSREQUEST']._serialized_end=3683
  _globals['_ALLELESINMULTIREGIONSINSAMPLESREQUEST']._serialized_start=3686
  _globals['_ALLELESINMULTIREGIONSINSAMPLESREQUEST']._serialized_end=4063
  _globals['_ALLELESRESPONSE']._serialized_start=4066
  _globals['_ALLELESRESPONSE']._serialized_end=4242
  _globals['_VARIANT']._serialized_start=4245
  _globals['_VARIANT']._serialized_end=4612
  _globals['_ALLELESWITHSTATSRESPONSE']._serialized_start=4615
  _globals['_ALLELESWITHSTATSRESPONSE']._serialized_end=4809
  _globals['_VARIANTWITHSTATS']._serialized_start=4812
  _globals['_VARIANTWITHSTATS']._serialized_end=5037
  _globals['_TOPNCHI2REQUEST']._serialized_start=5039
  _globals['_TOPNCHI2REQUEST']._serialized_end=5097
  _globals['_TOPNHWEREQUEST']._serialized_start=5099
  _globals['_TOPNHWEREQUEST']._serialized_end=5139
  _globals['_SAMPLESINREGIONREQUEST']._serialized_start=5142
  _globals['_SAMPLESINREGIONREQUEST']._serialized_end=5487
  _globals['_SAMPLESINMULTIREGIONSREQUEST']._serialized_start=5490
  _globals['_SAMPLESINMULTIREGIONSREQUEST']._serialized_end=5841
  _globals['_SAMPLESHOMREFREQUEST']._serialized_start=5844
  _globals['_SAMPLESHOMREFREQUEST']._serialized_end=6021
  _globals['_SAMPLESRESPONSE']._serialized_start=6024
  _globals['_SAMPLESRESPONSE']._serialized_end=6164
  _globals['_DENOVOREQUEST']._serialized_start=6167
  _globals['_DENOVOREQUEST']._serialized_end=6528
  _globals['_HETDOMINANTREQUEST']._serialized_start=6531
  _globals['_HETDOMINANTREQUEST']._serialized_end=6922
  _globals['_HOMRECESSIVEREQUEST']._serialized_start=6925
  _globals['_HOMRECESSIVEREQUEST']._serialized_end=7321
  _globals['_PRSREQUEST']._serialized_start=7323
  _globals['_PRSREQUEST']._serialized_end=7428
  _globals['_PRSRESPONSE']._serialized_start=7431
  _globals['_PRSRESPONSE']._serialized_end=7674
  _globals['_SAMPLESCORE']._serialized_start=7677
  _globals['_SAMPLESCORE']._serialized_end=7825
  _globals['_FSTATXREQUEST']._serialized_start=7828
  _globals['_FSTATXREQUEST']._serialized_end=7988
  _globals['_SEXMISMATCHRESPONSE']._serialized_start=7991
  _globals['_SEXMISMATCHRESPONSE']._serialized_end=8226
  _globals['_SAMPLESTAT']._serialized_start=8228
  _globals['_SAMPLESTAT']._serialized_end=8316
  _globals['_FSTATXRESPONSE']._serialized_start=8319
  _globals['_FSTATXRESPONSE']._serialized_end=8531
  _globals['_KINSHIPREQUEST']._serialized_start=8534
  _globals['_KINSHIPREQUEST']._serialized_end=8677
  _globals['_KINSHIPDUOREQUEST']._serialized_start=8679
  _globals['_KINSHIPDUOREQUEST']._serialized_end=8745
  _globals['_KINSHIPTRIOREQUEST']._serialized_start=8748
  _globals['_KINSHIPTRIOREQUEST']._serialized_end=8908
  _globals['_KINSHIPRESPONSE']._serialized_start=8911
  _globals['_KINSHIPRESPONSE']._serialized_end=9068
  _globals['_RELATEDNESS']._serialized_start=9070
  _globals['_RELATEDNESS']._serialized_end=9191
  _globals['_SAMPLEKINSHIPREQUEST']._serialized_start=9194
  _globals['_SAMPLEKINSHIPREQUEST']._serialized_end=9346
  _globals['_SAMPLEKINSHIPRESPONSE']._serialized_start=9349
  _globals['_SAMPLEKINSHIPRESPONSE']._serialized_end=9544
  _globals['_RELATEDNESSPERSAMPLE']._serialized_start=9547
  _globals['_RELATEDNESSPERSAMPLE']._serialized_end=9746
  _globals['_COUNTALLELESINREGIONREQUEST']._serialized_start=9749
  _globals['_COUNTALLELESINREGIONREQUEST']._serialized_end=10070
  _globals['_COUNTALLELESINREGIONINSAMPLESREQUEST']._serialized_start=10073
  _globals['_COUNTALLELESINREGIONINSAMPLESREQUEST']._serialized_end=10420
  _globals['_COUNTALLELESINBRACKETREQUEST']._serialized_start=10423
  _globals['_COUNTALLELESINBRACKETREQUEST']._serialized_end=10789
  _globals['_COUNTALLELESINBRACKETINSAMPLESREQUEST']._serialized_start=10792
  _globals['_COUNTALLELESINBRACKETINSAMPLESREQUEST']._serialized_end=11184
  _globals['_COUNTALLELESINMULTIREGIONSREQUEST']._serialized_start=11187
  _globals['_COUNTALLELESINMULTIREGIONSREQUEST']._serialized_end=11514
  _globals['_COUNTALLELESINMULTIREGIONSINSAMPLESREQUEST']._serialized_start=11517
  _globals['_COUNTALLELESINMULTIREGIONSINSAMPLESREQUEST']._serialized_end=11870
  _globals['_COUNTALLELESRESPONSE']._serialized_start=11873
  _globals['_COUNTALLELESRESPONSE']._serialized_end=12016
  _globals['_COUNTSAMPLESRESPONSE']._serialized_start=12019
  _globals['_COUNTSAMPLESRESPONSE']._serialized_end=12162
  _globals['_DNAERYSSERVICE']._serialized_start=16494
  _globals['_DNAERYSSERVICE']._serialized_end=21033
# @@protoc_insertion_point(module_scope)



"""Paginated variant query API for the Dnaerys client library.

``PaginatedQuery`` manages an internal buffer via ``collections.deque``,
refills transparently from the server, and serves fixed-size pages to the
caller.  ``skip`` and ``limit`` become invisible implementation details.
"""

from __future__ import annotations

from collections import deque
from collections.abc import Callable, Iterator
from dataclasses import dataclass
from types import TracebackType

from dnaerys._stream import VariantStream
from dnaerys._types import ResponseMetadata, Variant


@dataclass(frozen=True, slots=True)
class Page:
    """A single page of variant results from a paginated query.

    Parameters
    ----------
    variants : tuple[Variant, ...]
        The variants in this page.
    page_number : int
        1-based page number.
    """

    variants: tuple[Variant, ...]
    page_number: int


class PaginatedQuery:
    """Iterator that serves fixed-size pages of variants from a Dnaerys query.

    Manages an internal ``collections.deque`` buffer, refilling it
    transparently from the server via the provided *fetch* callable.
    Callers see clean, fixed-size pages without needing to know about
    ``skip``/``limit`` or cluster topology.

    Parameters
    ----------
    fetch : Callable[[int, int], VariantStream]
        A callable that takes ``(skip, limit)`` and returns a
        ``VariantStream``.
    page_size : int
        Number of variants per page (last page may be shorter).
    buffer_size : int
        Number of variants to request per server round-trip.

    Raises
    ------
    ValueError
        If *page_size* < 1 or *buffer_size* < *page_size*.
    """

    def __init__(
        self,
        fetch: Callable[[int, int], VariantStream],
        *,
        page_size: int,
        buffer_size: int = 5000,
    ) -> None:
        if page_size < 1:
            raise ValueError("page_size must be >= 1")
        if buffer_size < page_size:
            raise ValueError("buffer_size must be >= page_size")

        self._fetch = fetch
        self._page_size = page_size
        self._buffer_size = buffer_size
        self._buffer: deque[Variant] = deque()
        self._offset: int = 0
        self._page_number: int = 0
        self._exhausted: bool = False
        self._total_fetched: int = 0

        # Metadata accumulators (same pattern as VariantStream)
        self._incomplete_cluster: bool = False
        self._affected: bool = False
        self._elapsed_ms: int = 0
        self._elapsed_db_ms: int = 0
        self._node_ids: set[str] = set()

    @property
    def page_size(self) -> int:
        """Number of variants per page."""
        return self._page_size

    @property
    def buffer_size(self) -> int:
        """Number of variants requested per server round-trip."""
        return self._buffer_size

    @property
    def total_fetched(self) -> int:
        """Total number of variants served across all pages so far."""
        return self._total_fetched

    @property
    def has_more(self) -> bool:
        """Whether more pages may be available.

        Returns ``True`` if the buffer contains variants or the server has
        not yet been confirmed exhausted.  Returns ``False`` only after
        iteration is complete and the buffer is empty.

        .. note::
            Before the first :meth:`next_page` call, this property returns
            ``True`` even if the query matches zero variants — no server
            call has been made yet, so exhaustion is unknown.  The first
            :meth:`next_page` call triggers the first server round-trip;
            if it returns no variants, :exc:`StopIteration` is raised
            immediately and ``has_more`` becomes ``False``.
        """
        return not (self._exhausted and len(self._buffer) == 0)

    @property
    def metadata(self) -> ResponseMetadata:
        """Accumulated metadata from all server round-trips so far."""
        return ResponseMetadata(
            elapsed_ms=self._elapsed_ms,
            elapsed_db_ms=self._elapsed_db_ms,
            node_id=",".join(sorted(self._node_ids)) if self._node_ids else "",
            incomplete_cluster=self._incomplete_cluster,
            affected=self._affected,
        )

    def _accumulate_metadata(self, stream_metadata: ResponseMetadata) -> None:
        self._incomplete_cluster = self._incomplete_cluster or stream_metadata.incomplete_cluster
        self._affected = self._affected or stream_metadata.affected
        self._elapsed_ms = max(self._elapsed_ms, stream_metadata.elapsed_ms)
        self._elapsed_db_ms = max(self._elapsed_db_ms, stream_metadata.elapsed_db_ms)
        if stream_metadata.node_id:
            for part in stream_metadata.node_id.split(","):
                part = part.strip()
                if part:
                    self._node_ids.add(part)

    def _refill(self) -> None:
        stream = self._fetch(self._offset, self._buffer_size)
        new_variants = stream.to_list()
        self._accumulate_metadata(stream.metadata)
        if not new_variants:
            self._exhausted = True
            return
        self._buffer.extend(new_variants)
        self._offset += self._buffer_size

    def next_page(self) -> Page:
        """Return the next page of variants.

        Raises
        ------
        StopIteration
            When no more variants are available.
        """
        while len(self._buffer) < self._page_size and not self._exhausted:
            self._refill()

        if not self._buffer:
            raise StopIteration

        count = min(self._page_size, len(self._buffer))
        page_variants = [self._buffer.popleft() for _ in range(count)]
        self._page_number += 1
        self._total_fetched += count
        return Page(variants=tuple(page_variants), page_number=self._page_number)

    def __iter__(self) -> Iterator[Page]:
        return self

    def __next__(self) -> Page:
        return self.next_page()

    def __enter__(self) -> PaginatedQuery:
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        self._buffer.clear()
        self._exhausted = True
